/*-------------------------------------------------------------------------------------------------
  HIP Install: doInstall                                                                          
  action = portal.doInstall.action                                                                
                                                                                                 
  created 1999-12-17                                                                                      
  Author: Michael Gartler                                                                         
-------------------------------------------------------------------------------------------------*/
resdef ("v1.0/components/portal/langres/portal.lrc","port"); // language support
//-------------------------------------------------------------------------------------------------
function readFile(path) {
  //
  // reads the content of a file (e.g. content of install.js)
  //
  // In: 
  //  - path  : string containing the filename 
  //
  // Out: 
  //  - a string : the file content
  //
  var content;
  var f=new File(path);
  if(!f.open('r')) {
    return('Error: file '+path+' not found.'); }
  content=f.read(f.getLength());
  f.close();
  return content;
}
//-------------------------------------------------------------------------------------------------
function writeInstallResult(args) {
  //
  // writes the formatted result (+ buttons to status.log, license.txt and readme.txt if available) 
  // of an install session
  // 
  // In: param object with propertis {Name, Version, Status}
  //
  //   Name       : string containing name of Track (Package)
  //   Version    : string containing version of Track
  //   Status     : string; can have the following values describing the installation status of a package
  //                      'installed', 'failed', 'no_hwx', 'exists', 'uninstalled', 'ufail'
  //   status_log : filename of status.log (if exists)
  //   license_txt: filename of license.txt (if exists)
  //   readme_txt : filename of readme.txt (if exists)
  //
  // Out: writes the result
  //

  writeln('<TR>\
<TD NOWRAP><SPAN class="AttrName">'+args.Name+' ('+args.Version+')</SPAN></TD><TD>&nbsp;</TD>'); // package name + version
  write('<TD ALIGN="RIGHT" NOWRAP><SPAN class="AttrValue">...');
  switch (args.Status) {
    case 'installed'      : write(lrc("instDlgTxt1","port","installed")); break;      // install successful
    case 'failed'         : write(lrc("instDlgTxt2","port","failed")); break;         // install failed
    case 'uninstalled'    : write(lrc("instDlgTxt5","port","uninstalled")); break;    // uninstall successful
    case 'uninstall_error': write(lrc("instDlgTxt6","port","could not uninstall")); break; // uninstall failed
    default               : write('???'); break;      // unknown status: this should actually _never_ happen...
 }
  writeln('</SPAN></TD>');
  writeln('<TD>&nbsp;</TD><TD>');
  if (args.status_log!="")                                                            // BUTTON: status.log
    write('<INPUT TYPE="BUTTON" ONCLICK="opDlg(\''+args.status_log+'\',\'status_log\',{minWidth:\'780\'})" VALUE="status.log">');
  writeln('</TD><TD>&nbsp;</TD><TD>');
  if (args.license_txt!="")                                                           // BUTTON: license.txt
    write('<INPUT TYPE="BUTTON" ONCLICK="opDlg(\''+args.license_txt+'\',\'license_txt\')" VALUE="license.txt">');
  writeln('</TD><TD>&nbsp;</TD><TD>');
  if (args.readme_txt!="")                                                            // BUTTON: readme.txt
    write('<INPUT TYPE="BUTTON" ONCLICK="opDlg(\''+args.readme_txt+'\',\'readme_txt\')" VALUE="readme.txt">');
  writeln('</TD></TR>');
 }
//-------------------------------------------------------------------------------------------------
function writeResultPage(the_queue) {
  // 
  // displays detailed installation result page
  //
  // In: packs
  //
  // packs : string array containing the paths to the package directories
  //  
  var viewFileUrl="/;internal&action=portal.viewFile.action&Parameter=";
  var packs=the_queue.items;
  write('<TABLE BORDER="0">');
  
  for (var i=0; i<packs.length; i++) { // loop thru all packages
    var the_path=packs[i].path;
    var pkg_info=parsePkgXML({filename:the_path+'/pkg.xml'});// get info from pkg.xml
    var status_log=the_path+'/status.log';                   // build the complete path to the status.log file
    var Fstatus=new File(status_log);                        // create file object for status.log
    var license_txt=the_path+'/license.txt';                 // build the complete path to the license.txt file
    var Flicense=new File(license_txt);                      // create file object for license.txt
    var readme_txt=the_path+'/readme.txt';                   // build the complete path to the readme.txt file
    var Freadme=new File(readme_txt);                        // create file object for readme.txt
    var installed_xml=the_path+'/installed.xml';
    var Finstalled=new File(installed_xml);
    
    var statusFileUrl=Fstatus.exists()?viewFileUrl+escape(arrayToCommaseparated(["status.log",status_log])):"";
    var licenseFileUrl=Flicense.exists()?viewFileUrl+escape(arrayToCommaseparated(["license.txt",license_txt])):"";
    var readmeFileUrl=Freadme.exists()?viewFileUrl+escape(arrayToCommaseparated(["readme.txt",readme_txt])):"";

    if (Finstalled.exists()) {
      installedObj=parseInstalledXML({filename:installed_xml});
      var status=installedObj.Status;
      var timestamp=installedObj.Timestamp;}
    else
      var status='';
    
    if (status!="") {
      var param={ Name        : pkg_info.packageId.Name, 
                  Version     : pkg_info.packageId.Version, 
                  Status      : status,
                  status_log  : statusFileUrl,
                  license_txt : licenseFileUrl,
                  readme_txt  : readmeFileUrl };
      writeInstallResult(param); // write detailed installation results for a package
    }
    var xml='<?xml version="1.0"?>';
    /*
    if (status!='installed')
      writeFile(Finstalled,xml+'<Status></Status>\n<Timestamp></Timestamp>');*/
  }
  write('</TABLE>');
}
//-------------------------------------------------------------------------------------------------
function performAction(params) {
  //
  // In: params
  //  
  // params:
  //  - packagePath
  //  - action
  //  - username
  //  - password
  //

  var backup       = params.backup;
  var packagePath  = params.packagePath;
  var action       = params.action;
  var the_username = params.username;
  var the_password = params.password;
  var the_hostname = wavemaster.host;
  var the_port     = wavemaster.port;

  the_hostname=hw_getServerIPAddress();
  the_port=418;

  var install_js=packagePath+"/install.js";     // install.js
  var Finstall=new File(install_js);
  var uninstall_js=packagePath+"/uninstall.js"; // uninstall.js
  var Funinstall=new File(uninstall_js);

  var isRemove=true;
  var isBackup=backup=='backup_yes'?true:false;

  if (action=='backup') {
    action='uninstall';
    isRemove=false; }

//  if ((isRemove==false) && (isBackup==false))
//   action="";

  switch (action) {
    case 'install' ://===== I N S T A L L ============================================================
      if (Finstall.exists()) {                  // if install.js exists...
        var install_script=readFile(install_js); // read the install script

        var _hwsettings=HW_Settings;             // save HW_Settings before eval (eval seems to delete HW_Settings for some reason...)


        eval(install_script);                    // evaluate the install script code 
        HW_Settings=_hwsettings;                 // restore HW_Settings

        if (typeof(install)=='undefined') {      // function install() not found in file install.js
        }                                     // error handling if install undefined
        else {     
          //call function install()

          var inargs = { hwsettings  : HW_Settings,
                         hwhost      : the_hostname, 
                         hwport      : the_port, 
                         username    : the_username, 
                         password    : the_password, 
                         path        : packagePath+'/'}
          
          if (typeof HW_Product == "object") inargs.hwproduct = HW_Product;
          
          var result=install("",inargs); 
        } 
      }
      
      else { // install.js not found
      } break;
    case 'uninstall' : //===== U N I N S T A L L ======================================================
      if (Funinstall.exists()) {                       // if uninstall.js exists
        var uninstall_script=readFile(uninstall_js); // read the uninstall script
        var _hwsettings=HW_Settings;
        eval (uninstall_script);                     // evaluate the install script code
        HW_Settings=_hwsettings;
        if (typeof(uninstall)=='undefined') {        // function uninstall() not found in file uninstall.js
        } // error handling if uninstall undefined
        else {
          var result=uninstall("",{isRemove   : isRemove,
                                   isBackup   : isBackup,
                                   hwsettings : HW_Settings,
                                   hwhost     : the_hostname, 
                                   hwport     : the_port, 
                                   username   : the_username, 
                                   password   : the_password, 
                                   path       : packagePath+'/'});} }
      else {                                        // uninstall.js not found 
      } break; 
  }
}
//------------------------------------------------------------------------------------------------
function loadHiddenFrame() {
  if (navigator.appName.indexOf('Netscape')!=-1) {
    var _doc=document;
    var _x=_doc.progress.pageX; var _y=_doc.progress.pageY;
    var _h=_doc.progress.clip.height;
    _doc.lprogress.pageX=_x;_doc.lprogress.pageY=_y;
    _doc.lprogress.resizeTo(10,_h);
    _doc.lprogress.bgColor="'+progressBarColor+'";}
  document.f1.submit();
}
//------------------------------------------------------------------------------------------------
function clearDC_Cache() {
  //
  //
  //
  top.opener.location.replace("/;internal&action=dc.clearcache.action");
  top.close();
}
//------------------------------------------------------------------------------------------------
function Queue_toCommaseparated() {
  //
  //
  //
  var arr=[this.username,this.password,this.backup];
  var install_arr=[];
  var uninstall_arr=[];
  var result="";

  for(var i=0; i<this.items.length; i++) {
    var the_action=this.items[i].action;
    var the_path  =this.items[i].path;
    switch (the_action) {
      case 'install'   : install_arr.push(the_path,the_action); break;
      case 'uninstall' : uninstall_arr.push(the_action,the_path); break;
      case 'backup'    : uninstall_arr.push(the_action,the_path); break;} 
  }
  uninstall_arr.reverse();
  arr=arr.concat(install_arr,uninstall_arr);
  result=arrayToCommaseparated(arr); 
  return (result);
}
//------------------------------------------------------------------------------------------------
function Queue_toCommaseparated() {
  //
  //
  //
  var arr=[this.username,this.password,this.backup];
  var result="";

  for(var i=0; i<this.items.length; i++) {
    var the_action=this.items[i].action;
    var the_path  =this.items[i].path;
    var the_title =this.items[i].title;
    arr.push(the_path,the_action,the_title); }
  result=arrayToCommaseparated(arr); 
  return (result);
}
//------------------------------------------------------------------------------------------------
function Queue_splitInstallUninstall() {
  //
  //
  //
  var arr=[this.username,this.password,this.backup];
  var install_arr=[];
  var uninstall_arr=[];
  var result="";

  for(var i=0; i<this.items.length; i++) {
    var the_action=this.items[i].action;
    var the_path  =this.items[i].path;
    var the_title =this.items[i].title;
    switch (the_action) {
      case 'install'   : install_arr.push(the_path,the_action,the_title); break;
      case 'uninstall' : uninstall_arr.push(the_title,the_action,the_path); break;
      case 'backup'    : uninstall_arr.push(the_title,the_action,the_path); break;} 
  }
  uninstall_arr.reverse();
  arr=arr.concat(uninstall_arr,install_arr);

  this.items=[];
  for (var i=3; i<arr.length; i=i+3)
    this.items.push({path:arr[i],action:arr[i+1],title:arr[i+2]});
  
  result=arrayToCommaseparated(arr); 
  return (result);
}
//------------------------------------------------------------------------------------------------
function Queue(queue) {
  //
  //
  //
  var qarr=commaseparated(queue);           // retrieve package sortorder

  this.toCommaseparated      = Queue_toCommaseparated;
  this.splitInstallUninstall = Queue_splitInstallUninstall;
  this.username              = qarr.shift();               // remove username from queue
  this.password              = qarr.shift();               // remove password from queue
  this.backup                = qarr.shift();               // remove backup from queue
  this.items                 = [];

  for(var i=0; i<qarr.length; i=i+3) {
   var the_path=qarr[i];
   var the_action=qarr[i+1];
   var the_title=qarr[i+2];

   if (the_action!='nochange')
    this.items.push({path:the_path,action:the_action,title:the_title}); }
}
//------------------------------------------------------------------------------------------------
function setExpire() {
  //
  //
  //
    var HW_Date=new Date("December 1, 1990 11:11:11");
    response.field.Expires=HW_Date.toGMTString();
    HW_Date=new Date();
    HW_Date.setTime(HW_Date.getTime()-2000);
    response.field["Last-Modified"]=HW_Date.toGMTString();
    response.field.Pragma="no-cache";
    response.field["Cache-Control"]="no-cache";
}
//------------------------------------------------------------------------------------------------
function translateAction(action) {
  //
  //
  //
  var res='';
  switch (action) {
    case 'install'   : res=lrc('instDlgTxt14','port','Installing'); break;
    case 'uninstall' : res=lrc('instDlgTxt15','port','Uninstalling'); break;
    case 'backup'    : res=lrc('instDlgTxt16','port','Creating Backup of'); break;  
  }
  return res;
}
//------------------------------------------------------------------------------------------------
function progressUI_content(params) {

  var segment_size     = params.segment_size;
  var par              = params.par;
  var new_par          = params.new_par;
  var progressBarWidth = params.progressBarWidth;
  var progressBarColor = params.progressBarColor;
  var first_action     = translateAction(params.first_action);
  var first_pkg        = params.first_pkg;

  writeln('<SCRIPT>\
document.f1.target="hiddenframe";\
var prog=0;\
var segment='+segment_size+';\
var sortorder="'+par+'";\
var url="'+new_par+'";');
  writeln(loadHiddenFrame);
  writeln('</SCRIPT>');
  
  writeln('<INPUT TYPE="HIDDEN" VALUE="'+new_par+'" NAME="INSTALL_PARAMETERS">\
<BR>\
<TABLE BORDER=0 WIDTH="100%">\
<TR><TD><SPAN CLASS="AttrName">'+lrc("instDlgCap7","port","Installation Progress")+'</SPAN></TD></TR>\
<TR><TD ALIGN="CENTER"><TABLE BORDER="1" WIDTH="'+progressBarWidth+'" CELLSPACING="0" CELLPADDING="0">\
<TR>');
  if (segment_size==0)
    writeln('<TD BGCOLOR="'+progressBarColor+'">');
  else
    writeln('<TD>');

  write('<DIV ID="progress" STYLE="position:');
  if (request.agent.name=='MSIE')
    write('absolute; width:10; background-color:'+progressBarColor+'"></DIV>&nbsp;');
  else
    write('relative;">&nbsp;</DIV>');
  writeln('</TD></TR></TABLE>');

  if (request.agent.name=='Netscape') 
    writeln('<LAYER ID="lprogress"></LAYER>');

writeln('</TD></TR>\
<TR><TD><SPAN CLASS="AttrName">'+lrc("instDlgTxt7","port","Current Action")+'</SPAN></TD></TR>\
<TR><TD ALIGN="CENTER"><TABLE BORDER="1" WIDTH="400" CELLSPACING="0" CELLPADDING="0">\
<TR><TD>\
<DIV ID="status" STYLE="position:relative; left:0; top:0"><SPAN CLASS=\'AttrValue\'>'+first_action+'<BR><NOBR>'+first_pkg+'</NOBR></SPAN>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<BR></DIV>\
<LAYER ID="lstatus"></LAYER></TD></TR>\
</TABLE></TR></TD></TR>\
</TABLE><BR>');
}
//-------------------------------------------------------------------------------------------------
function updateProgressBar(params) {
  //
  //
  //
  var progressBarWidth=params.progressBarWidth;
  var progressBarColor=params.progressBarColor;
  var next_action     =translateAction(params.next_action);
  var next_pkg        =params.next_pkg;
  var par             =params.par;
  
  writeln('<HTML>\
<HEAD><SCRIPT>\
function onload_handler() {\
  var progress=parent.progressview;\
  var doc=progress.document;\
  progress.prog+=progress.segment;\
  if (progress.prog>'+(progressBarWidth-4)+')\
   progress.prog='+(progressBarWidth-4)+';\
  if (navigator.appName.indexOf("Microsoft")!=-1){\
   parent.progressview.document.all.status.innerHTML="<SPAN CLASS=\'AttrValue\'>'+next_action+'<BR><NOBR>'+next_pkg+'</NOBR><BR><BR><BR></SPAN>";\
   parent.progressview.document.all.progress.style.backgroundColor="'+progressBarColor+'";\
   parent.progressview.document.all.progress.style.width=progress.prog;}\
  if (navigator.appName.indexOf("Netscape")!=-1){\
   x=doc.progress.pageX;y=doc.progress.pageY;\
   h=doc.progress.clip.height;w=doc.progress.clip.width;\
   doc.lprogress.pageX=x;doc.lprogress.pageY=y;\
   doc.lprogress.resizeTo(progress.prog,h);\
   doc.lprogress.bgColor="'+progressBarColor+'";\
   x=doc.status.pageX; y=doc.status.pageY;\
   h=doc.status.clip.height; w=doc.status.clip.width;\
   doc.lstatus.pageX=x; doc.lstatus.pageY=y;\
   doc.lstatus.resizeTo(w,h);\
   doc.status.document.write("");\
   doc.status.document.close();\
   doc.lstatus.document.write("<SPAN CLASS=\'AttrValue\'>'+next_action+'<BR><NOBR>'+next_pkg+'</NOBR></SPAN>");\
   doc.lstatus.document.close();}\
 document.f1.submit();\
}\
</SCRIPT>\
</HEAD>\
<BODY ONLOAD="onload_handler()">\
<FORM NAME="f1" ACTION="'+params.doInstallAction+'" METHOD="POST">\
<INPUT TYPE="HIDDEN" VALUE="'+escape(par)+'" NAME="INSTALL_PARAMETERS">\
</FORM>\
</BODY></HTML>');
}
//-------------------------------------------------------------------------------------------------
function loadFinishPage(params) {
  //
  //
  //
  writeln('<HTML><HEAD>\
 <SCRIPT>\
 function onloadHandler() {\
  document.f1.INSTALL_PARAMETERS.value="finish"+parent.progressview.sortorder;\
  document.f1.submit();}\
 </SCRIPT>\
 <BODY ONLOAD="onloadHandler()">\
 <FORM NAME="f1" ACTION="'+params.doInstallAction+'" METHOD="POST" TARGET="_top">\
  <INPUT TYPE="HIDDEN" NAME="INSTALL_PARAMETERS" VALUE="">\
 </FORM>\
 </BODY>\
 </HEAD></HTML>');
}
//-------------------------------------------------------------------------------------------------
function checkUser(username,password) {
  //
  //
  //
  var out=server.hw_DAP().userInGroups({uid:username,gids:['system']});
  if (out.error[0].code==2)  
    return false;
  /*
  var serva=new HW_API_Server({host:wavemaster.host,port:wavemaster.port});

  out=serva.identify({username:username,password:password});
 if (out.error.error())
  { writelnError('identify error');
    return false;}
    */
  return true;
}
//-------------------------------------------------------------------------------------------------
function createDC_Collection() {
  //
  //
  //
  out=server.object({objectidentifier:'hw_documentclasses'})
  if (out.error.error()) {
    //    writelnError('doc classes not found');
    var insobj=new HW_API_Object(
      new HW_API_Attribute("Type","Document"),
      new HW_API_Attribute("Name","hw_documentclasses"),
      new HW_API_Attribute("DocumentType","collection"),
      new HW_API_Attribute("PresentationHints","Hidden"),
      new HW_API_Attribute("Title","en:Document Classes"),
      new HW_API_Attribute("Title","ge:Dokumentklassen"));
    out=server.insert({"object":insobj,"parameters":new HW_API_Object(new HW_API_Attribute("Parent","rootcollection"))});
    if (out.error.error())
      for (x in out.error)
        writelnError('errror:',out.error[x].description("en"));
  }
}
//-------------------------------------------------------------------------------------------------
function HW_portal_doInstall() {
  //
  // installs/uninstall tracks (packages)
  //
  // In: (doInstall is called by the install.html dialog)
  //   - USERNAME (string)
  //   - PASSWD (string)
  //   - selected_new (string) : the paths (filesystem) to packages after user selection
  //   - selected_old (string) : the paths (filesystem) to packages before user selection
  //
  // Out:
  //
  var installScript     ="install.js";      // name of the install script file
  var uninstallScript   ="uninstall.js";    // name of the uninstall script file
  var statusFile        ="status.log";      // name of the install status file
  var licenseFile       ="license.txt";     // name of track (package) license file
  var readmeFile        ="readme.txt";      // name of the readme file
  var viewFileUrl       ="/;internal&action=portal.viewFile.action&Parameter="; 
                                            // action to display the content of files (in filesystem). 
                                            // Needed e.g. to display content of status.log 
                                            // (the parameter contains the filename)
  var doInstallAction  ="/;internal&action=portal.doInstall.action";
  var progressBarWidth=400;
  var progressBarColor="blue";

//***** DIALOG SPECS *****************************************************************************************
  var dlg_spec={                            // dlg specs for installation progress page
    caption   : lrc("instDlgCap5","port","Installation Progress"),
    opdlgfunc : 1, setexpire:1,
    formaction: doInstallAction,
    onload    : "loadHiddenFrame()"};
  var dlg_spec2={                           // dlg specs for installation result page
    caption   : lrc("instDlgCap4","port","Install Result"),
    script    : clearDC_Cache,
    buttons   : {"Close":{exec:"clearDC_Cache()"},"  Help  ":{func:"help",funcext:"installresults"}},
    opdlgfunc : 1};
  var dlg_spec3={                           // dlg specs for invalid username
    caption   : "Access Denied",
    buttons   : {"Close":{exec:"top.close()"},"  Help  ":{func:"help"}},
    opdlgfunc : 1, setexpire:1};
//************************************************************************************************************

  createDC_Collection();

  var obj=KeyValueField(request.content.read(-1)); // get data
  var par=unescape(obj.INSTALL_PARAMETERS);
   
  // parameter value (commaseparated list of values)
  // pos 1: ["finish"|"progress"]
  // - commaseparated values -
  // pos 2: username
  // pos 3: password
  // pos 4: ["backup_yes"|"backup_no"]
  // pos 5: path to package1
  // pos 6: action to perform with package 1
  // pos 7: (all other packages, see pos4, pos5)

//
// *** Display Installation Result ***
//
if (par.indexOf('finish')!=-1) { 
par=par.substr(6);            // get rid of 'finish' at the beginning of the param string
    var qobj= new Queue(par); // paramstr -> Queue
writeln('<SCRIPT>\
if (navigator.appName.indexOf("Netscape")!=-1) {\
window.setResizable(true);\
window.outerHeight=600;\
window.outerWidth=800;\
}\
</SCRIPT>');
    HW_DC_dialogHeader(dlg_spec2); 
    writeResultPage(qobj);              // display the installation result (in package sortorder)
    HW_DC_dialogFooter(dlg_spec2); 
  }
  else
// -----------------------------------------------------------------------Display Installation Progress Page
    if (par.indexOf('progress')!=-1) {   
      par=par.substr(8);                  // get rid of 'progress' at the beginning of parameter string
      var qobj= new Queue(par);
      var cu=checkUser(qobj.username,qobj.password);
      var segment_size=(qobj.items.length)!=0?Math.ceil((progressBarWidth-2)/(qobj.items.length)):0; 
                                      // compute the length of a progress bar segment
      new_par=escape(qobj.splitInstallUninstall());
      par    =escape(par);
    
      var first_pkg    = qobj.items[0].title;
      var first_action = qobj.items[0].action;

      if (!cu) {
        HW_DC_dialogHeader(dlg_spec3);
        writeln('<SPAN Class="Caption">Invalid username or password.</SPAN>');
        HW_DC_dialogFooter(dlg_spec3); }
      else {
        HW_DC_dialogHeader(dlg_spec); 
        var retobj=new HW_SSJ_ReturnObject({description:"install error"});
        progressUI_content({par:par,
                            new_par:new_par,
                            progressBarWidth:progressBarWidth,
                            progressBarColor:progressBarColor,
                            segment_size:segment_size,
                            first_action:first_action,
                            first_pkg:first_pkg});
        HW_DC_dialogFooter(dlg_spec); }
    }
    else { 
//
// *** load finish page ***
//
      var qobj= new Queue(par);
      if (qobj.items.length==0) {                                 // check if queue is empty -> finish
        setExpire();                                             // don't use browser cache 
        loadFinishPage({doInstallAction:doInstallAction});}
        // load the finish page of the installation process
//
// *** process the packages ***
//
      else { 
        var obj=qobj.items.shift();
        var pkg         = obj.path;           // the path to the package 
        var action      = obj.action;         // the action to perform with this package
        var username    = qobj.username;
        var password    = qobj.password;
        var backup      = qobj.backup;
        var next_pkg    = qobj.items.length>0?qobj.items[0].title:"";
        var next_action = qobj.items.length>0?qobj.items[0].action:"";

        performAction({packagePath : pkg,
                       action      : action,
                       backup      : backup,
                       username    : username, 
                       password    : password});   

        par=qobj.toCommaseparated();

        setExpire(); // dont'use browser cache

        updateProgressBar({progressBarWidth:progressBarWidth,
                           progressBarColor:progressBarColor,
                           next_action:next_action,
                           next_pkg:next_pkg,
                           par:par,
                           doInstallAction:doInstallAction});

      }
    }
  
}
