/* -----------------------------------------------------------------------------
  This program reads the Hyperwave product information, converts it to XML,  and 
  saves it under the name specified. 
  UNIX: The product information is stored in the file 'product' in the home 
        directory.

  Windows NT/2000: The product information is stored in the System Registry.
  --------------------------------------------------------------------------- */


/* -----------------------------------------------------------------------------
  Common part for UNIX and Windows
  Holds: * Definition of Option Parser
         * Check for operating system
  --------------------------------------------------------------------------- */
var x = new OptionParser(argv,{"-file":1}) ;

while (x.more()) {
  switch (x.option()) {
  case 1: // -file
    if (x.more() && !x.nextIsOption()) {
      filename = x.parameter() ;
    }
    else {
      write("-file requires parameter\n") ;
      exit(1);
    }
    break;
  case -1 :
    write ("Error: unknown option\n"); writeHelp();
    exit();
    break ;
  case -2 :
    write ("Error: ambiguous commandline option:", x.ambiguous(),"\n") ;
    exit();
    break ;
  default:
    write ("Internal error\n") ;
    exit();
    break;
  }
}

if (environment['os'])
  var isUnix = false;
else
  var isUnix = true;

//for test purposes
//var isUnix = true;

/* -----------------------------------------------------------------------------
  UNIX part
  Checks if product file is already XML formatted, converts it to XML, if 
  necessary, and stores it under the given name
  --------------------------------------------------------------------------- */
if (isUnix) {
  var homepath = environment['HOME'];
  var oldproductsfile = new File(homepath+'/products');
  if (!oldproductsfile.open('r')) {
    writelnError("Can't find product information file 'products' in directory "+homepath+".");
    exit(1);
  }
  var lineentry = oldproductsfile.readln();
  if (!lineentry.match(/<?xml[^<>]*?>/)) { // product file is not XML formatted -> convert it
    var outfile = '<?xml version="1.0"?>';
    outfile += '\n<HWPR>\n';
    var tagname = lineentry.match(/<([^<>]*)>/);
    if (tagname)
      tagname = tagname[1];
    outfile += lineentry+'\n';
    while (!oldproductsfile.eof()) {
      var lineentry = oldproductsfile.readln();
      var tagname = lineentry.match(/<([^<>]*)>/);
      tagname ? tagname = tagname[1] : tagname = '';
      switch (tagname) {
      case 'version':
        outfile += lineentry+'</version>\n';
        break;
      case 'release_date':
        outfile += lineentry+'</release_date>\n';
        break;
      case 'install_date':
        outfile += lineentry+'</install_date>\n';
        break;
      case 'install_path':
        outfile += lineentry+'</install_path>\n';
        break;
      case 'platform':
        outfile += lineentry+'</platform>\n';
        break;
      default:
        outfile += lineentry + '\n';
      }
    }
    outfile += '</HWPR>';
  }
  else { // product file is already XML -> copy file
    outfile = lineentry + oldproductsfile.read(oldproductsfile.getLength());
  }
  oldproductsfile.close();
  var out = new File(filename);
  if (!out.open('w')) {
    writeln("Can't write file "+filename+".");
    exit(1);
  }
  else {
    out.writeln(outfile);
    out.close();
  }
}
else {
/* -----------------------------------------------------------------------------
   Windows part:
   * Store Hyperwave product information in file 'regdump.tmp' using regedit.
   * Check if file is in ASCII format (Windows 2000 writes Unicode files) and 
     convert it to ASCII if necessary
   * write XML file
  --------------------------------------------------------------------------- */
  var command = 'regedit /e regdump.tmp HKEY_LOCAL_MACHINE\\SOFTWARE\\Hyperwave\\Products';
  var status = system(command);
  if (status) {
    writelnError('Could not read registry'); 
    exit(1);
  }

  var oldproductsfile = new File('regdump.tmp');
  if (!oldproductsfile.open('r')) {
    writelnError("Can't read product information from registry."); // happens if no product information available 
    var out = new File(filename);
    if (!out.open('w')) {
      writeln("Can't write file "+filename+".");
      exit(1);
    }
    else {
      out.writeln('<?xml version="1.0"?>');
      out.writeln('<HWPR>');
      out.writeln('</HWPR>');
      out.close();
    }
    exit(0);
  }
  var firstline = oldproductsfile.readln();
  if (firstline.indexOf('') == 0)   // file is unicode file -> must be converted it to ASCII
    var file_is_ascii = false;

  var filecontent = '';
  while (!oldproductsfile.eof()) {
    var line = oldproductsfile.readln();
    filecontent += line+'\n';
  }
  oldproductsfile.close();
  oldproductsfile.remove();
  if (!file_is_ascii) // unicode to ASCII conversion
    filecontent = filecontent.split('\x00').join('');
  
  filecontent = filecontent.substring(filecontent.indexOf('HKEY_LOCAL_MACHINE\\SOFTWARE\\Hyperwave\\Products\\')-1,
                                      filecontent.length);
  filecontent = filecontent.replace(/HKEY_LOCAL_MACHINE\\SOFTWARE\\Hyperwave\\Products\\/g,'');
  filecontent = ' '+filecontent;

  var index = 0;
  var position = filecontent.indexOf('\n',index+1);
  var line = '';

  var xmlroot = {};
  do {
    line = filecontent.substring(index+1,position);
    if (line.match(/\S/)) {
      switch(line[0]) {
      case '[':
        var xpatharray = line.substring(1,line.indexOf(']')).split('\\');
        var curobj = xmlroot;
        for (var id in xpatharray) {
          if (!curobj[xpatharray[id]]) {
            curobj[xpatharray[id]] = {};
          }
          curobj = curobj[xpatharray[id]];
        }
        break;
      case '"':
        var keyvaluepair = line.match(/"([^"]+)"="([^"]+)/); //"]) {
        curobj[keyvaluepair[1]] = keyvaluepair[2];
        break;
      default:
        writelnError('Unknown entry in Windows Registry.');
      }
    } 
    index = position;
    position = filecontent.indexOf('\n',index+1);
  } while(position >= 0);


  function writetree(theobject,indent,status) {
    for (var id in theobject) {
      if (typeof theobject[id] == 'object') {
        switch (status) {
        case 'Service Packs':
          
          out.writeln(indent+'<service_pack>'+id);
          writetree(theobject[id],indent+'  ',id,'Service Packs');
          out.writeln(indent+'</service_pack>');
          break;
        case 'Modules':
          out.writeln(indent+'<module>'+id);
          writetree(theobject[id],indent+'  ',id,'Modules');
          out.writeln(indent+'</module>');
          break;
        default:
          if (id != 'Service Packs' && id != 'Modules') 
            out.writeln(indent+'<product>'+id);
          writetree(theobject[id],indent+'  ',id,'Product');
          if (id != 'Service Packs' && id != 'Modules') 
            out.writeln(indent+'</product>');
        }
      }
      else
        out.writeln(indent+'<'+id.toLowerCase()+'>'+theobject[id]+'</'+id.toLowerCase()+'>');
    }
  }
  
  var out = new File(filename);
  if (!out.open('w')) {
    writeln("Can't write file "+filename+".");
    exit(1);
  }
  else {
    out.writeln('<?xml version="1.0"?>');
    out.writeln('<HWPR>');
    writetree(xmlroot,'','product');
    out.writeln('</HWPR>');
    out.close();
  }
}
