/* -----------------------------------------------------------------------------
  This program writes the Hyperwave product information.
  UNIX: The product information is stored in the file 'product' in the home 
        directory.

  Windows NT/2000: The product information is stored in the System Registry.
  --------------------------------------------------------------------------- */


/* -----------------------------------------------------------------------------
  Common part for UNIX and Windows
  Holds: * Definition of Option Parser
         * Check for operating system
  --------------------------------------------------------------------------- */
var x = new OptionParser(argv,{"-file":1}) ;

while (x.more()) {
  switch (x.option()) {
  case 1: // -file
    if (x.more() && !x.nextIsOption()) {
      filename = x.parameter() ;
    }
    else {
      write("-file requires parameter\n") ;
      exit(1);
    }
    break;
  case -1 :
    write ("Error: unknown option\n"); writeHelp();
    exit();
    break ;
  case -2 :
    write ("Error: ambiguous commandline option:", x.ambiguous(),"\n") ;
    exit();
    break ;
  default:
    write ("Internal error\n") ;
    exit();
    break;
  }
}

if (environment['os'])
  var isUnix = false;
else
  var isUnix = true;

/* -----------------------------------------------------------------------------
  UNIX part: Copy XML file to home directory
  --------------------------------------------------------------------------- */
if (isUnix) {  // take file and save it under the name product in the home directory
  var homepath = environment['HOME'];
  
  var xmlfile = new File(filename);
  if (!xmlfile.open('r')) {
    writelnError("Can't read temporary information file "+filename+".");
    exit(1);
  }
  var filecontent = xmlfile.read(xmlfile.getLength());
  xmlfile.close();

  // the products file has to be written in a special file format -> convert XML file to XML 
  // file in that special format
  filecontent = filecontent.replace(/<product>\s*/g,'<product>');
  filecontent = filecontent.replace(/<service_pack>\s*/g,'<service_pack>');
  filecontent = filecontent.replace(/<module>\s*/g,'<module>');
  filecontent = filecontent.replace(/<version>\s*(.*)\s*<\/version>/g,'<version>$1<\/version>');
  filecontent = filecontent.replace(/<release_date>\s*(.*)\s*<\/release_date>/g,'<release_date>$1<\/release_date>');
  filecontent = filecontent.replace(/<install_date>\s*(.*)\s*<\/install_date>/g,'<install_date>$1<\/install_date>');
  filecontent = filecontent.replace(/<install_path>\s*(.*)\s*<\/install_path>/g,'<install_path>$1<\/install_path>');
  filecontent = filecontent.replace(/<platform>\s*(.*)\s*<\/platform>/g,'<platform>$1<\/platform>');

  // write products file
  var productsfile = new File(homepath+'/products');
  if (!productsfile.open('w')) {
    writelnError("Can't write product file in directory "+homepath+".");
    exit(1);
  }
  productsfile.write(filecontent);
  productsfile.close();
  xmlfile.remove();
}
else {
/* -----------------------------------------------------------------------------
   Windows part: 
   * Create registry file and use regedit to store the information in the 
     registry.
  --------------------------------------------------------------------------- */
  // create registry file

  if (typeof HW_XML== 'undefined')  // include XML library if it's not already included
    include('xmllib.js');
  var xmlfile = new HW_XML();
  var status = xmlfile.toObject({file:filename});
  if (status.error()) {
    writelnError(status[0].description('en'));
    exit(1);
  }
  var outfile = new File('hyperwavetmp.reg');
  if (!outfile.open('w')) {
    writelnError("Can't write temporary registry file");
    exit(1);
  }
  outfile.writeln('REGEDIT4');
  outfile.writeln('\n[HKEY_LOCAL_MACHINE\SOFTWARE\Hyperwave\Products]');

  var productsstatus = {};
  var productslist = xmlfile.findEntities({name:'product'});
  for (var productsloop in productslist.result) {
    var entity = productslist.result[productsloop];
    for (var id in entity.contents) {
      if (typeof entity.contents[id] != 'object') {
        productname = entity.contents[id].replace(/\t|\n|\r/g,'');
        outfile.writeln('\n[HKEY_LOCAL_MACHINE\\SOFTWARE\\Hyperwave\\Products\\'+productname+']');
      }
      else {
        switch (entity.contents[id].name) {
        case 'service_pack':
          if (productsstatus[productname]) {
            if (!productsstatus[productname].sps) {
              productsstatus[productname].sps = true;
              outfile.writeln('\n[HKEY_LOCAL_MACHINE\\SOFTWARE\\Hyperwave\\Products\\'+productname+'\\Service Packs]');
            }
          }
          else {
            productsstatus[productname] = {modules:false, sps:true};
            outfile.writeln('\n[HKEY_LOCAL_MACHINE\\SOFTWARE\\Hyperwave\\Products\\'+productname+'\\Service Packs]');
          }
          for (var id2 in entity.contents[id].contents) {
            if (typeof entity.contents[id].contents[id2] != 'object') {
              var modulename = entity.contents[id].contents[id2].replace(/\t|\n|\r/g,'');
              outfile.writeln('\n[HKEY_LOCAL_MACHINE\\SOFTWARE\\Hyperwave\\Products\\'+productname+'\\Service Packs\\'+modulename+']');
            }
            else {
              var keyname = entity.contents[id].contents[id2].name;
              var keyvalue = entity.contents[id].contents[id2].contents[0].replace(/\t|\n|\r/g,'');
              outfile.writeln('"'+keyname+'"="'+keyvalue+'"');
            }
          }
          break;
        case 'module':
          if (productsstatus[productname]) {
            if (!productsstatus[productname].modules) {
              productsstatus[productname].modules = true;
              outfile.writeln('\n[HKEY_LOCAL_MACHINE\\SOFTWARE\\Hyperwave\\Products\\'+productname+'\\Modules]');
            }
          }
          else {
            productsstatus[productname] = {modules:true, sps:false};
            outfile.writeln('\n[HKEY_LOCAL_MACHINE\\SOFTWARE\\Hyperwave\\Products\\'+productname+'\\Modules]');
          }
          
          for (var id2 in entity.contents[id].contents) {
            if (typeof entity.contents[id].contents[id2] != 'object') {
              var modulename = entity.contents[id].contents[id2].replace(/\t|\n|\r/g,'');
              outfile.writeln('\n[HKEY_LOCAL_MACHINE\\SOFTWARE\\Hyperwave\\Products\\'+productname+'\\Modules\\'+modulename+']');
            }
            else {
              var keyname = entity.contents[id].contents[id2].name;
              var keyvalue = entity.contents[id].contents[id2].contents[0].replace(/\t|\n|\r/g,'');
              outfile.writeln('"'+keyname+'"="'+keyvalue+'"');
            }
          }
          break;
        default:
          var keyname = entity.contents[id].name;
          var keyvalue = entity.contents[id].contents[0].replace(/\t|\n|\r/g,'');
          outfile.writeln('"'+keyname+'"="'+keyvalue+'"');
        }
      }
    }
  }
  outfile.close();
  var command = 'regedit /s /i hyperwavetmp.reg';
  var status = system(command);
  if (status) {
    writelnError('Could not update registry');
    exit(1);
  }
  outfile.remove();
  var oldxmlfile = new File(filename);
  oldxmlfile.remove();
}

