/** ****************************************************************************
 * Hyperwave Information Portal                                                *
 * Package Install Script                                                      *
 * @author: Gartler Michael                                                    *
 *******************************************************************************/

var _version='Version 1.1, June 8th, 2000'; // the version of install.js

//-------------------------------------------------------------------------------------------------
function writeHelp() {
  //
  // writes help text for install.js (when executed from command line via hwjs)
  //
  // In: -
  // Out: help text 
  //
 writeln('\n\
Hyperwave Information Portal ('+_version+')\nPackage Install Script\n\n\
Usage: hwjs install.js [options] \n\n\
Options\n\t-hwhost name \t\t(opt) name of Hyperwave server host\n\
\t-hwport port \t\t(opt) port number to connect to\n\
\t-user username\t\t(req) specify username non-interactively\n\
\t-password password\t(req) specify password non-interactively\n\n'); 
}
//------------------------------------------------------------------------------------------------------------------
function importHWXFile (params) {
  //
  // imports a hwxfile into db with hwximport tool
  //
  // In: params object
  //     attributes:
  //        - res_name    : hwx filename
  //        - res_dest    : parent collection for hwx
  //        - res_path    : path to hwx file
  //        - inargs     
  //             - hwhost : server name
  //             - hwport : portnumber
  //        - the_username: username
  //        - the_password: password
  //
  // Out: result object
  //      attributes
  //        - result [true,false] : true if no errors occured
  //        - statusLog           : string containing details about all actions or errors
  //
  var res_name     = params.res_name;      // hwx filename
  var res_dest     = params.res_dest;      // parent collection for hwx (destination)
  var res_path     = params.res_path;      // path to hwx file
  var inargs       = params.inargs;        // inargs.hwhost, inargs.hwport
  var the_username = params.the_username;  // username
  var the_password = params.the_password;  // password
  var statusLog='';                        
  var res=true;

  statusLog+='HWX File:\t'+res_name+'\n';             // *** status.log ***
  statusLog+='Destination:\t'+res_dest+'\n';          // *** status.log ***
  var slash=res_path==""?"":"/";

  var exec='hwximport -replace always -hwhost '+inargs.hwhost+' -hwport '+inargs.hwport+' -identify "'+the_username+' '+the_password+'" -hwxfile '+res_path+res_name+' -parent '+res_dest;         
                                                      // hwximport command line

  var sysres=system (exec);                           // execute hwximport
  if (sysres==0) {                                            // check result of hwximport
   statusLog+='Installed:\tyes';                              // *** status.log ***
    }
  else {                                                      // hwximport error; something went wrong... 
   res=false;
   statusLog+='Installed:\tno\nError:\t\thwximport failed (errorcode: '+sysres+')';}

return ({result:res,statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function createUserGroup(params) {
  //
  // creates an user group with HWDAP
  //
  // In: params object
  //     attributes
  //        - gid         : group name 
  //        - parentgid   : name of parent group
  //        - Description : descriptive text for group
  //
  // Out: result object
  //      attributes
  //        - result [true,false] : true if no errors occured
  //        - statusLog           : string containing details about all actions or errors
  //
  var gid          = params.gid;
  var parentgid    = params.parentgid;
  var statusLog    = '';
  var res          = true;
  var Description  = params.Description;

  statusLog+='Group:\t\t'+gid+'\n';             // *** status.log ***
  statusLog+='Parentgroup:\t'+parentgid+'\n';   // *** status.log ***
  statusLog+='Created:\t';

  out=server.hw_DAP().getGroup({gid:gid});      // check if group exists
  if (out.error.error()) {
   out=server.hw_DAP().addGroup({gid:gid,parentgid:parentgid,groupattributes:new HW_API_Object(new HW_API_Attribute("Descr",Description))});                               // insert group
  if (out.error.error()) {
    statusLog+='no (could not create group)'+gid+':\n';
   for (var x in out.error) 
     statusLog+='\t'+out.error[x].description("en"); 
   }
  else
    statusLog+='yes';
  }
  else 
  statusLog+='no (group already exists.)';
  return ({result:res,statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function createCollection(params) {
  //
  // creates a collection in db
  //
  // In: params object
  //     attributes:
  //        - res_name    : collection name
  //        - res_dest    : parent collection
  //        - attributes  : attributes for new collection
  //
  // Out:
  //      attributes
  //        - result [true,false] : true if no errors occured
  //        - statusLog           : string containing details about all actions or errors
  //
  var coll_name   = params.res_name;
  var parent      = params.res_dest;
  var attributes  = params.attributes;
  var statusLog   = '';
  var res         = true;

  statusLog+='Collection:\t'+coll_name+'\n';    // *** status.log ***
  statusLog+='Parent:\t\t'+parent+'\n';         // *** status.log ***
  statusLog+='Created:\t';

  out=server.object({objectidentifier:coll_name});
  if (out.error.error()) {
    var insobj=new HW_API_Object(new HW_API_Attribute("Type","Document"),new HW_API_Attribute("DocumentType","collection"),new HW_API_Attribute("Name",coll_name),new HW_API_Attribute("Title","en:"+coll_name));
    for (var x in attributes) {
      insobj.insert(new HW_API_Attribute(x,attributes[x])); } 
   out=server.insert({"object":insobj,"parameters":new HW_API_Object(new HW_API_Attribute("Parent",parent))});
   if (out.error.error()) {
     res=false;
     statusLog+='no';
    for (var x in out.error) 
     statusLog+=out.error[x].description("en"); }
    else
    statusLog+='yes'; 
   }
  else
   statusLog+='no (collection already exists)';
  return ({result:res,statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function unzipFile(params) {
  //
  // unzips a zip archive file into a directory
  //
  // In: params object
  //     attributes
  //        - source      : name of zip archive
  //        - destination : destination directory for zip archive
  //
  // Out: 
  //      attributes
  //        - result [true,false] : true if no errors occured
  //        - statusLog           : string containing details about all actions or errors
  //
  var source       = params.source.toLowerCase();
  var Fsource      = new File(source); 
  var destination  = params.destination.toLowerCase();

  var statusLog    = '';
  var res          = true;

  statusLog+='Unzip File:\tsource: '+source+'\n\t\tdestination: '+destination+'\n\t\tresult: ';

  if (!Fsource.exists()) {
   res=false;
   statusLog+='source file not found.';
   return ({result:res,statusLog:statusLog}); }

  var exec='unzip -o '+source+' -d '+destination;
  var sysres=system(exec);

  if (sysres==0) 
   statusLog+='ok';
  else {
   statusLog+='failed (error code:'+sysres+')';
   res=false; }

  return ({result:res,statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function copyFile (params)
{
  var source       = params.source;
  var Fsource      = new File(source); 
  var destination  = params.destination.toLowerCase();
  var index        = source.lastIndexOf('/');
  var filename     = index != -1 ? source.substring(index+1) : source;

  var statusLog    = '';
  var res          = true;

  statusLog+='Copy File:\tsource: '+source+'\n\t\tdestination: '+destination+'\n\t\tresult: ';

  if (!Fsource.exists()) 
  {
   res=false;
   statusLog+='source file not found.';
   return ({result:res,statusLog:statusLog}); 
  }

  if (!Fsource.copy(destination+'/'+filename))
  {
    statusLog+='failed (cannot copy file).';
    return ({result:false,statusLog:statusLog}); 
  }
  
  statusLog+='ok.';
  return({result:true, statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------
function createInclude(params) {
  //
  // author: mwieder
  //
  var package_name = params.package_name;
  var package_version = params.package_version;
  var is_place_include = params.is_place_include;
  var destination = params.destination;
  var include_file = params.include_file;
  var install_path = params.install_path;
  if (destination.substr(-include_file.length) == include_file)
    destination = destination.substring(0, destination.length - include_file.length);
  if (destination.substr(-1) == '/')
    destination = destination.substring(0, destination.length-1);

  var statusLog = 'Add Include:\t' + include_file + '\n';

  // copy file into destination directory
  var out = copyFile({source:install_path + include_file, destination:HW_Settings.HTML_DIR + '/' + destination});
  if (!out.result) {
//    statusLog += 'Cannot copy "' + include_file + '" into directory "' + destination + '"\nInstalled:\tno';
    return out;
  }

  // create include directive in 'hwincludes.html'
  var VERSION_DIR = HW_Settings.VERSION_DIR || "v1.0";
  var hwincludes_name = HW_Settings.HTML_DIR + '/' + VERSION_DIR + '/hwincludes.html';

  var file = new File(hwincludes_name);
  if(!file.open('r')) {
    statusLog += 'Cannot open "' + hwincludes_name + '" for reading\nInstalled:\tno';
    return ({result:false, statusLog:statusLog});
  }

  var full_include = destination + '/' + include_file;
  if (is_place_include)
    var lines = "%%# PKGINC (PLACE)(" + package_name + " [" + package_version + "])(" + full_include + ") %%\n" +
                "%%include \"" + full_include + "\"%%\n";
  else
    var lines = "%%# PKGINC (JS)(" + package_name + " [" + package_version + "])(" + full_include + ") %%\n" +
                "<server src=\"file://" + full_include + "\">\n";

  var content = file.read(file.getLength());
  file.close();

  if (content.indexOf(lines) != -1)
    statusLog += 'File is already included\n';
  else {
    file = new File(hwincludes_name);
    if(!file.open('w')) {
      statusLog += 'Cannot open "' + hwincludes_name + '" for writing\nInstalled:\tno';
      return ({result:false, statusLog:statusLog});
    }

    if (content.substr(-1) != '\n')
      content += '\n';

    file.write(content + lines);
    file.close();
  }

  statusLog += 'Installed:\tyes';
  return ({result:true, statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function addZero(x) {
  x='0'+x;
  x=x.substr(-2);
  return(x);
}
//------------------------------------------------------------------------------------------------------------------
function getDateString(dobj) {
  //
  //
  //
  var year=dobj.getFullYear()+"";
  year=year.substr(2);
  var month=dobj.getMonth()+1;
  month=addZero(month);
  var day=dobj.getDate();
  day=addZero(day);
  var hour=dobj.getHours();
  hour=addZero(hour);
  var minute=dobj.getMinutes();
  minute=addZero(minute);
  var second=dobj.getSeconds();
  second=addZero(second);

  var res=month+day+year+'.'+hour+minute+second;
  return res;
}
//------------------------------------------------------------------------------------------------------------------
function create_saveDir(params) {
  //
  // creates a backup directory ('save_dir') in a specified directory. 
  //
  // In: params object
  //   - path : the parent directory of the save_dir
  //
  // Out: status object
  //   - result  : [true|false] (true if operation was successful)
  //   - statusLog : details, error messages etc.
  //
  var statusLog='Backup:\t'
  var the_path=params.path;
  var dobj=new Date();
  var the_name='backup.'+getDateString(dobj);
  var dir_obj=new Directory(the_path+the_name);
  var out=dir_obj.create();
  if(!out)
    return({result:false,statusLog:statusLog+' could not create directory '+dir_obj.name+'.\n',backupdir:''});
  else
    return({result:true,statusLog:statusLog+' directory '+dir_obj.name+' created.\n',backupdir:the_path+the_name});
}
//------------------------------------------------------------------------------------------------------------------
function backupDirectory(params) {
  //
  // moves directory to the backup dir (savedir)
  //
  // In : params object
  //      - dir_name
  //      - savedir_name
  //
  // Out: status object
  //   - result  : [true|false] (true if operation was successful)
  //   - statusLog : details, error messages etc.
  //

  var dir_name=HW_Settings.HTML_DIR+'/'+params.dir_name;
  var statusLog='Backup Directory:\t'+dir_name+' ... ';

  var file_obj = new File(dir_name);
  if (file_obj.exists())
  {
    var dir_obj=new Directory(dir_name);
    
    var savedir_name=params.savedir_name+'/'+params.dir_name.replace(/\//g,'_');
    var res=true;

    if(!dir_obj.move(savedir_name)) {
      statusLog+='failed.\n\t\t\tError: could not move '+dir_name+' to '+savedir_name;
      return({result:false,statusLog:statusLog}); }
    else
      statusLog+='ok.';
  }
  else
  {
    statusLog+='Cannot find directory. Backup not possible/necessary.';
    res=true;
  }

  return({result:res,statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function findUniqueName(basedir,name) {
  //
  //
  //
  name=name.substr(0,250);
  var fobj=new File(basedir+'/'+name);
  var index=1;
  var name2="";
  while (fobj.exists()) {
    name2=name+'~'+index;
    fobj=new File(basedir+'/'+name2);
    index++ }

  if (name2=="")
    return basedir+'/'+name;
  else
    return basedir+'/'+name2;
}
//------------------------------------------------------------------------------------------------------------------
function objectExists(objid)
{
  var out = server.object({objectidentifier:objid});
  if (out.error.error()) {
    return false;
  }
  return true;
}
//------------------------------------------------------------------------------------------------------------------
function backupObject(params) {
  //
  // deletes an object from the database; saves all data modified after a given data (timestamp) to the savedir
  //
  // In: params object
  //      - object
  //      - timestamp
  //      - backupdir
  //
  // Out: status object
  //   - result  : [true|false] (true if operation was successful)
  //   - statusLog : details, error messages etc.
  //

  var the_object=params.object;
  var the_backupdir=params.backupdir;
  var the_backupdestination=the_object.replace(/\//g,'_');

  var uniqueBackupDirName=findUniqueName(the_backupdir,the_backupdestination);
  var dobj=new Directory(uniqueBackupDirName);
  out=dobj.create();
  if (!out)
    writeln('could not create dir '+dobj.name);

  var res=true;
  var statusLog='Backup Collection:\t'+the_object+' ... '; 

  if (objectExists(the_object))
  {
    var exec='APIhwdownload -hwhost '+params.hwhost+' -hwport '+params.hwport+' -identify "'+params.username+' '+params.password+'" -parent '+the_object+' -targetdir '+the_backupdir+'/'+the_backupdestination;
    
    var sysres=system (exec);                           // execute hwdownload
          //    writelnError('--->ready !!');
    if (sysres!=0) {                                    // check result of hwdownload
      res=false;
      statusLog+='failed.\n\tError: could not APIhwdownload ';
      return({result:res,statusLog:statusLog}); 
    }
    statusLog+='ok.';
  }
  else
  {
    res=true;
    statusLog+='Cannot find Collection. Backup not possible/necessary.';
  }

  return ({result:res,statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function backupFile(params) {
  var filename         = params.filename;
  var savedir_name     = params.savedir_name;

  var statusLog = 'Backup File:\t\t' + HW_Settings.HTML_DIR + '/' + filename + ' ... ';

  var file = new File(HW_Settings.HTML_DIR + '/' + filename);

  if (file.exists())
  {
    var index = filename.lastIndexOf('/');
    var fname = index == -1 ? filename : filename.substring(index+1);
    
    if (!file.move(savedir_name + '/' + fname)) {
      statusLog += 'failed.\n\t\tError: Cannot move "' + filename + '" to "' + savedir_name + '".';
      return ({result:false, statusLog:statusLog});
    }
  }
  else
  {
    statusLog += 'Cannot find file. Backup not possible/necessary.';
    return ({result:true, statusLog:statusLog});
  }

  statusLog += 'ok.';
  return ({result:true, statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function checkProduct (params)
{
  var statusLog = "";
  var product_obj = params.product || {};
  var installed_product = params.installed_product || {};

  if (typeof product_obj != "undefined")
  {
    var the_operator = product_obj.Operator || "ge";
    if (typeof product_obj.SP != "string") installed_product.SP = "";  // no service pack dependency defined
    var compare_obj = eval('portal_'+the_operator+'(installed_product,product_obj)');
    
    if (typeof compare_obj!="undefined" && !compare_obj.result)
    {
      the_product = product_obj.Product;
      the_product_version = product_obj.Version;
      the_product_SP = product_obj.SP;
      switch (compare_obj.reason)
      {
        case "Name" : 
        {
          product_str = "This package requires " + the_product;
          if (the_product_version && the_product_SP)
            product_str += ' (V'+the_product_version.replace(/v/ig,"")+', SP '+the_product_SP+')';
          else
          {
            if (the_product_version)
              product_str += ' (V'+the_product_version.replace(/v/ig,"")+')';
            else
              if (the_product_SP)
                product_str += "(SP "+the_product_SP+")";
          }
          product_str += ".\n";
          break;
        }
         
        case "Version" : 
        {
          product_str = "This package requires Version " + the_product_version.replace(/v/ig,"");
          if (the_product_SP)
            product_str += " (SP"+the_product_SP+")";
          product_str += ".\n";
          break;
        }
         
        case "SP" : 
        {
          product_str = "This package requires SP "+the_product_SP+".\n";
          break;
        }
        default: break;
      }
      return {result:false, statusLog:product_str};
    }
    else
      return {result:true, statusLog:""};
  }
  return {result: false, statusLog: statusLog};
}
//------------------------------------------------------------------------------------------------------------------
function portal_convertVersion (version_str)
{
  if (typeof version_str != "string" ||
    version_str == "")
    return 0;

  version_str = version_str.replace(/v/i,"");
  if (version_str.indexOf('_')!=-1)
    var arr = version_str.split('_');
  else
    var arr = version_str.split('.');
  var ret = 0;
  for (var i=0; i<3; i++)
  {
    ret *= 100;
    if (typeof(arr[i])!="undefined")
      ret += parseInt(arr[i]);
  }
  return ret;
}

//------------------------------------------------------------------------------------------------------------------
function portal_eq (obja, objb) 
{
  var obja = obja || {};
  var objb = objb || {};

  if (obja.Product != objb.Product) 
    return {result:false, reason:"Name"};
  if (portal_convertVersion(obja.Version) != portal_convertVersion(objb.Version)) 
    return {result:false, reason:"Version"};
  if (portal_convertVersion(obja.SP) != portal_convertVersion(objb.SP)) 
    return {result:false, reason:"SP"};

  return {result:true};
}

//------------------------------------------------------------------------------------------------------------------
function portal_ge (obja, objb) 
{
  var obja = obja || {};
  var objb = objb || {};

  if (obja.Product != objb.Product) 
    return {result:false, reason:"Name"};
  if (portal_convertVersion(obja.Version) < portal_convertVersion(objb.Version)) 
    return {result:false, reason:"Version"};
  if (portal_convertVersion(obja.SP) < portal_convertVersion(objb.SP)) 
    return {result:false, reason:"SP"};

  return {result:true};
}

//------------------------------------------------------------------------------------------------------------------
function portal_extractInstallInfo (productXML)
{
  var retobj;
  var installed_products=productXML.findEntities({name:'product'}); // get 'Package' entity

        // loop through found products
  for (var i=0; i<installed_products.result.length; i++)
  {
    var contents = installed_products.result[i].contents;
          // look for products of interest
    if (typeof contents[0]=="string" &&
      ((contents[0].search(/^\s*(Hyperwave Information Server)\s$/)!=-1 && (typeof retobj!="object" || typeof retobj["Product"]!="string")) || 
        contents[0].search(/^\s*(Hyperwave Information Portal)\s$/)!=-1))  // remove leading and trailing spaces
    {
      retobj = {};
      retobj["Product"] = RegExp.$1;  // product name
      retobj["product"] = RegExp.$1;  // product name
      for (var j=1; j<contents.length; j++)
      {
        if (contents[j].name != "service_pack")  // store prodct information in return object
        {
          retobj[contents[j].name] = contents[j].contents[0]; // version, release_date
          if (contents[j].name=="version") retobj["Version"] = contents[j].contents[0];
        }
        else
        {
                // store service_pack information in object retobj.service_pack
          retobj[contents[j].name] = {};
          for (var k=0; k<contents[j].contents.length; k++)
          {
            if (typeof contents[j].contents[k] == "object")
            {
              retobj[contents[j].name][contents[j].contents[k].name] = contents[j].contents[k].contents[0];
                    // store version of service_pack into property SP to make
                    // comparisons with Dependency information in packages easier
              if (contents[j].contents[k].name == "version")
                retobj["SP"] = contents[j].contents[k].contents[0]; // service pack version
            }
          }
        }
      }
      if (retobj["Product"]=="Hyperwave Information Portal") break;  // product found -> exit loop
    }
  }
  return typeof retobj == "object" ? retobj : null;
}

//------------------------------------------------------------------------------------------------------------------
function portal_setServicePackInfo (productsinfo, servicepackinfo)
{
  var spobj = (typeof servicepackinfo.value[0] == "object" ? servicepackinfo.value[0] : servicepack.value);
        // replaces placeholder "%%Today%%" with the current date
        // necessary for tag "Install_Date"
  for (var k=1; k<spobj.contents.length; k++)
  {
    if (spobj.contents[k].contents[0]=="%%Today%%")
      spobj.contents[k].contents[0] = outDate(new Date(),"YYYY-MM-DD");
  }

  var installed_products = productsinfo.findEntities({name:'product'});  // get installed products
        // find the product to which the service pack belongs
  for (var i=0; i<installed_products.result.length; i++)
  {
    var contents = installed_products.result[i].contents;
          // remove leading and trailing spaces
    var found = (typeof contents[0]=="string" ? contents[0].search(/^\s*(.*)\s*$/) : -1);  
    if (found != -1)
    {
      var product = RegExp.$1;
      if (product == servicepackinfo.attributes["Product"])
      {
        var set = false;  
        for (var j=1; j<contents.length; j++)
        {
                // if a service pack already exists replace this information
          if (contents[j].name == "service_pack")  
          {
            contents[j] = spobj;
            set = true;
            break;  // servive pack found -> exit loop
          }
        }
              // if no service pack has existed yet add a new entry
        if (!set)
          contents[contents.length] = spobj;
        break;  // product found -> exit loop
      }
    }
  }
//  recur(productsinfo.xmlroot,"");
  return productsinfo;
}
//------------------------------------------------------------------------------------------------------------------
function hw_readGlobalProductInfo(parameters) 
{
  var thefilename = parameters.file;
  var prefix = parameters.prefix;
  var command = 'hwjs '+prefix+'hwpread.js -file '+thefilename;
  system(command);
  
  var xmlobj = new HW_XML();
  xmlobj.toObject({file:thefilename});
  return xmlobj;
}
//------------------------------------------------------------------------------------------------------------------
function hw_writeGlobalProductInfo(xmlobj,parameters) 
{
  var prefix = parameters.prefix;
  var thefilename = parameters.file;
  var status = xmlobj.toString({filename:thefilename});
  var command = 'hwjs '+prefix+'hwpwrite.js -file '+thefilename;
  system(command);
}
//------------------------------------------------------------------------------------------------------------------
//  function recur (obj,indent)
//  {
//    for (var x in obj)
//      if (typeof obj[x] == "object") 
//      {
//        writelnError(indent + x);
//        recur(obj[x], indent + "  ");
//      }
//      else
//        writelnError(indent + x + ' : ' + obj[x]);
//  }
//------------------------------------------------------------------------------------------------------------------
function install(api,inargs) {
  //
  // installs a track (package)
  //
  // In: api, inargs
  // 
  // inargs:
  //  - path       : the path to the package directory
  //  - hwhost     : the name of the hyperwave server (default=localhost)
  //  - hwport     : the port number (default=418)
  //  - username   : the username
  //  - password   : the password
  //  - hwsettings : HW_Settings object
  // 
  // Out: result
  //
  // result:
  //  - status (string)     :  status of installation; [installed,failed,exists] 
  // 

  var pkgInfo         = parsePkgXML({filename:inargs.path+"pkg.xml"});    // read info from pkg.xml 
  var the_Name        = pkgInfo.packageId.Name;                           // the package name
  var the_Version     = pkgInfo.packageId.Version;                        // the package version
  var the_ForceBackup = pkgInfo.packageId.ForceBackup;                    // change for compatibility to HIP 1.0
  if (typeof(the_ForceBackup)=="undefined")
    the_ForceBackup = "true";

  var the_Title     = pkgInfo.Title[0].value;                           // the package title
  var the_Resources = pkgInfo.Resource;                                 // the package resources
  var the_Dependency = pkgInfo.Dependency;
  if (typeof the_Dependency == "undefined")
    the_Dependency = [{attributes:{Product:"Hyperwave Information Portal",Version:"1.0"}}]
  
  var the_Register  = pkgInfo.Register;              
  var res_path      = inargs.path;                                      // path to unzipped resources

  var the_username  = inargs.username;                                  // username
  var the_password  = inargs.password;                                  // password
  var the_hostname  = inargs.hwhost;                                    // hostname
  var the_port      = inargs.hwport;                                    // port number

  var pathInstalled = inargs.path+'installed.xml';                      // path to 'installed' file
  var pathStatus    = inargs.path+'status.log';                         // path to 'status.log' file
  var Finstalled    = new File(pathInstalled);                          // 'installed'
  var Fstatus       = new File(pathStatus);                             // 'status.log'

  var inst;                                                             // installation status
  var inst_Date     = new Date();
  var the_error     = "";

  var xml='<?xml version="1.0"?>\n';

  var HW_Settings=inargs.hwsettings;

  var statusObj={result:true,statusLog:'Hyperwave Information Portal '+_version+'\nPackage Installation Status Report\n'+inst_Date.toLocaleString()+'\n\nPackage:\t'+the_Name+'\nVersion:\t'+the_Version+'\n\nResources:\n'};

  var installed_product;
  if (typeof the_Dependency != "undefined")
  {
    for (var i=0; i<the_Dependency.length; i++) 
    {
      if (typeof(the_Dependency[i].attributes.Product)!="undefined")
      {
        var prefix = (inargs.path=="" ? "./" : inargs.path);
        installed_product = portal_extractInstallInfo(hw_readGlobalProductInfo({file:HW_Settings.HTML_DIR+"/"+(HW_Settings.VERSION_DIR || "v1.0")+"/lib/products.xml",prefix:prefix}));
        if (installed_product == null) 
          installed_product = {Product:"Hyperwave Information Portal", Version:"1.0"}
        out = checkProduct({product: the_Dependency[i].attributes, installed_product: installed_product});
        if (!out.result)
        {
          statusObj.result=false;
          statusObj.statusLog+='\n\nError:'+out.statusLog; 
        }
        break;
      }
    }
  }

  if ((statusObj.result) && typeof the_Register != "undefined")
  {
    if (typeof installed_product != "undefined")
    {
      var prefix = (inargs.path=="" ? "./" : inargs.path);
      installed_product = portal_extractInstallInfo(hw_readGlobalProductInfo({file:HW_Settings.HTML_DIR+"/"+(HW_Settings.VERSION_DIR || "v1.0")+"/lib/products.xml",prefix:prefix}));
      if (installed_product == null) 
        installed_product = {Product:"Hyperwave Information Portal", Version:"1.0"}
    }

          // extract service pack version of the new package
    var register = (typeof the_Register[0][0] == "object" ? the_Register[0][0].value : the_Register[0].value);
    var sp;
    for (var i=0; i<register.length; i++)
    {
      if (typeof register[i] == "object")
      {
        if (register[i].name == "service_pack")
          for (var j=1; j<register[i].contents.length; j++)
          {
            if (register[i].contents[j].name == "version")
              sp = register[i].contents[j].contents[0];
          }
      }
    }
    if (typeof sp == "string")
    {
      if (portal_convertVersion(installed_product.SP)>portal_convertVersion(sp))
      {
        statusObj.result=false;
        statusObj.statusLog+='\n\nError: Cannot install SP '+sp+'. Reason: SP '+installed_product.SP +' already installed.'; 
      }
    }
  }

  if (statusObj.result)
  {

          // check if package is already installed ---------------------------------------------------------------------
    if (Finstalled.exists()) { 
      var installedObj=parseInstalledXML({filename:pathInstalled});
//    var the_Timestamp=install_obj.Timestamp;                              // get timestamp
//    the_Timestamp=outDate(dateConv(the_Timestamp,HWDate,true),HWDate.full); 
      if (installedObj.Status=='installed')
        return({status:'exists'});
    }
    
          // create backup
    if (the_ForceBackup=="true")
    {
      var obj_toRemove  = pkgInfo.Uninstall;                                // the objects to remove 
      if (typeof(obj_toRemove)=='undefined')
      {
        the_error='Error in the Uninstall Information (no objects defined to remove)';
        statusObj.statusLog+=the_error;
        statusObj.result = false;
      }
      else
      {
        out=create_saveDir({path:res_path}); // create backup directory 
        statusObj.statusLog+=out.statusLog;
        var the_backupdir=out.backupdir;
        if (!out.result) {
          the_error='Error: could not create backup directory';
          statusObj.statusLog+=the_error;
          statusObj.result = false;
        }
        else
        {
          for (var i=0; i<obj_toRemove.length; i++)                   // loop thru all objects to remove
          {
            var obj=obj_toRemove[i].attributes.Object;                  // name of the object to remove
            var type=obj_toRemove[i].attributes.Type;
            
            switch(type) {
              case 'his' : 
                out=backupObject({ object:obj, 
                                   backupdir:the_backupdir,
                                   hwhost:the_hostname,
                                   hwport:the_port,
                                   username:the_username,
                                   password:the_password}); 
                
                if (!out.result)
                  statusObj.result=false;
                statusObj.statusLog+='\n'+out.statusLog; 
                break;
                
              case 'dir' :
                out=backupDirectory({ dir_name:obj, 
                                      savedir_name:the_backupdir}); 
                
                if (!out.result)
                  statusObj.result=false;
                statusObj.statusLog+='\n'+out.statusLog;
                break;
                
              case 'file' :
                out = backupFile({ filename:obj, 
                                   savedir_name:the_backupdir});
                
                if (!out.result)
                  statusObj.result=false;
                statusObj.statusLog+='\n'+out.statusLog;
                break;
            }
          }
        }
      }
    }
    
    if (typeof the_Resources != "undefined")
    {        
      for (var i=0; i<the_Resources.length; i++) {                  // loop thru all hwx resources
        var res_name=the_Resources[i].attributes.Name;              // name of the hwx resource
        var res_dest=the_Resources[i].attributes.Destination;       // destination for the resource
        var res_type=the_Resources[i].attributes.Type;              // resource type
        
        switch (res_type) {
          case 'hwx' :
            out=importHWXFile({res_name:res_name,res_dest:res_dest,res_path:res_path,the_username:the_username,the_password:the_password,inargs:inargs}); 
            if (!out.result)
              statusObj.result=false;
            statusObj.statusLog+='\n\n'+out.statusLog; break;
          case 'group' :
            var parentgid=the_Resources[i].attributes.Parentgroup;
            var Description=the_Resources[i].attributes.Description;
            out=createUserGroup({gid:res_name,parentgid:parentgid,Description:Description}); 
            if (!out.result)
              statusObj.result=false;
            statusObj.statusLog+='\n\n'+out.statusLog; break;
          case 'collection' :
            var Attributes=the_Resources[i].value.attributes;
            out=createCollection({res_name:res_name,res_dest:res_dest,attributes:Attributes});
            if (!out.result)
              statusObj.result=false;
            statusObj.statusLog+='\n\n'+out.statusLog; break;
          case 'zip' :
            out=unzipFile({source:res_path+res_name,destination:HW_Settings.HTML_DIR+'/'+res_dest});
            if (!out.result)
              statusObj.result=false;
            statusObj.statusLog+='\n\n'+out.statusLog; break;
          case 'file' :
            out=copyFile({source:res_path+res_name,destination:HW_Settings.HTML_DIR+'/'+res_dest});
            if (!out.result)
              statusObj.result=false;
            statusObj.statusLog+='\n\n'+out.statusLog; break;
          case 'placeinclude' :
            out = createInclude({package_name:the_Name, package_version:the_Version, is_place_include:true,
                                 install_path:res_path, include_file:res_name, destination:res_dest});
            if (!out.result)
              statusObj.result=false;
            statusObj.statusLog+='\n\n'+out.statusLog; break;
          case 'jsinclude' :
            out = createInclude({package_name:the_Name, package_version:the_Version, is_place_include:false,
                                 install_path:res_path, include_file:res_name, destination:res_dest});
            if (!out.result)
              statusObj.result=false;
            statusObj.statusLog+='\n\n'+out.statusLog; break;
        }
      }
    }
  }

  if (statusObj.result)
  {
          // register new product information
    if (typeof the_Register != "undefined")
    {
      var prefix = (inargs.path=="" ? "./" : inargs.path);
      var installed_products = hw_readGlobalProductInfo({file:HW_Settings.HTML_DIR+"/"+(HW_Settings.VERSION_DIR || "v1.0")+"/lib/products.xml",prefix:prefix});
      var pinfo = the_Register[0];
//      recur(pinfo," ");
      portal_setServicePackInfo(installed_products, pinfo);
      hw_writeGlobalProductInfo(installed_products,{file:HW_Settings.HTML_DIR+"/"+(HW_Settings.VERSION_DIR || "v1.0")+"/lib/products.xml",prefix:prefix});
    }
  }
  
  out=server.dbStat();
  var timestamp=out.object.LocalTime;

  if (statusObj.result) 
   file_result=writeFile(Finstalled,xml+'<Status>installed</Status>\n<Timestamp>'+timestamp+'</Timestamp>'); 
  else
   file_result=writeFile(Finstalled,xml+'<Status>failed</Status>\n<Timestamp>'+timestamp+'</Timestamp>'); 
                                                             

  file_result+=writeFile(Fstatus,statusObj.statusLog);                    // write 'status.log'

  if (statusObj.result)
   return({status:"installed"});                             
  else
   return({status:"failed"});
}
//------------------------------------------------------------------------------------------------------------------
function hw_internal_DAP()
{
  if (!this.hwDAP) this.hwDAP=new HW_DAP(this);
  return this.hwDAP;
}
//===================================================================================================
if (typeof(server)=='undefined') { // Execution context is HWJS... 
  //
  // check if execution context is hwjs (there's no server object in hwjs)
  // this is needed to determine wheter the install scripts is called from commandline or via templates
  //

  HW_Settings={HTML_DIR:"../../../../"};

  var dir = new Directory(HW_Settings.HTML_DIR);
  if (!dir.open())
  {
    writeln('\n\nError: Cannot include necessary files. Cannot open directory '+HW_Settings.HTML_DIR+'.\n\n');
  }
  else
  {
      var version_dir = "";
      for (;;)
      {
        var name = dir.read();
        if (!name)
          break;
        if (name.search(/^v.\..+$/)!=-1 && name!="v4.1") 
        {
          version_dir = name;
          break;
        }
      }
      dir.close();
      
      if (version_dir == "")
      {
        writeln('\n\nError: Cannot include necessary files. Cannot find version directory.\n\n');
      }
      else
      {
        HW_Settings.VERSION_DIR = version_dir;

        include(HW_Settings.HTML_DIR+'/'+HW_Settings.VERSION_DIR+"/lib/datefunctions.js");   
        include(HW_Settings.HTML_DIR+'/'+HW_Settings.VERSION_DIR+"/lib/xmllib.js");                               // xml parser
        include(HW_Settings.HTML_DIR+'/'+HW_Settings.VERSION_DIR+"/components/portal/code/install_lib.js");       // required lib functions      
        var args=parseParams();
        
        if (args.username=='')
        { writeln('\nPlease specify an username.'); writeHelp(); exit(); }  // Error: no username specified
        
        if (args.password=='')
        { writeln('\nPlease specify a password.'); writeHelp(); exit();}    // Error: no password specified

        server=new HW_API_Server({host:args.hwhost,port:args.hwport});
        out=server.identify({username:args.username,password:args.password});
        if (out.error.error()) {
          writeln('\n\nError: Identification failed.\n\n');
          for (var x in out.error) 
            writeln(out.error[x].description("en"));
          exit(); }
        
        var hwdap_obj=new HW_DAP(server);
        server.hw_DAP=hw_internal_DAP;
        
        args.path="";
        args.hwsettings=HW_Settings;
        
        var result=install("",args);
        
        switch (result.status) {
          case 'installed' :
            writeln('\nPackage installed. See file "status.log" for further details.\n'); break;
          case 'failed' :
            writeln('\nPackage not installed. There was an error. See file "status.log" for further details.\n'); break;
          case 'exists' :
            writeln('\nThis package has already been installed.\n'); break;
        }
      }
  }
        //   if (result.file_error!="") writeln('Errors:\n\n'+result.file_error); // show filesystem errors
}
