//
//  function parsePkgXML
//  function parseParams
//  function writeFile
//

function parsePkgXML(params) {
  //
  // retrieves the information stored in pkg.xml
  //
  // In: params object 
  //      - filename: full path to pkg.xml
  //
  // Out: object
  //      - packageId   : the package id object
  //         - Name    : the package name, e.g. HW_PortalTrackBookmark
  //         - Version : the package version, e.g. "Beta"
  //      - Title[0].value       : package title
  //      - Description[0].value : package description
  //      - Dependency[1..n].attributes : the dependencies
  //             -Name    : the name of the package that is required to use this package
  //             -Version : the version of the package that is required to use this package
  //
  //      - Resource[1..n].attributes
  //             -Name
  //             -Destination
  //      - Result
  //             -Error
  //             -Description

var the_filename=params.filename;                // the file containing the xml information (pkg.xml)
var pkgXML=new HW_XML();
var status=pkgXML.toObject({file:the_filename}); // read & parse xml data

if (status.error()) 
  return({packageId  :{Name:'',Version:'xml_error'},
          Title      :[{value:the_filename}], 
          Description:[{value:status[0].description("en")}],
          Resource   :[],
          Result     :{Error:true,Description:the_filename}});


var myPkg=pkgXML.findEntities({name:'Package'}); // get 'Package' entity
var pkgId=myPkg.result[0].attributes;            // store package attributes
                                                   // create package id object (Name,Version)
var pkgInfo={};                                  // this object holds the complete information about the package
pkgInfo.packageId=pkgId;
for (var i=0; i<myPkg.result[0].contents.length; i++) 
{
        // go thru the package contents: <Title>,<Description>,<Dependency>,<Resource>
  if (typeof(pkgInfo[myPkg.result[0].contents[i].name])=='undefined') // create a new member in pkgInfo
    if (myPkg.result[0].contents[i].name!="Register")  // in case of Register information store the whole content
      pkgInfo[myPkg.result[0].contents[i].name]=[{value:myPkg.result[0].contents[i].contents[0],attributes:myPkg.result[0].contents[i].attributes}];
    else
      pkgInfo[myPkg.result[0].contents[i].name]=[{value:myPkg.result[0].contents[i].contents,attributes:myPkg.result[0].contents[i].attributes}];
  else
  {
    if (myPkg.result[0].contents[i].name!="Register")  // in case of Register information store the whole content
      pkgInfo[myPkg.result[0].contents[i].name].push({value:myPkg.result[0].contents[i].contents[0],attributes:myPkg.result[0].contents[i].attributes});
    else
      pkgInfo[myPkg.result[0].contents[i].name].push({value:myPkg.result[0].contents[i].contents,attributes:myPkg.result[0].contents[i].attributes});
  }
}
pkgInfo.Result={Error:false,Description:''};
return pkgInfo;
}
//-------------------------------------------------------------------------------------------------
function parseInstalledXML(params) {
var the_filename=params.filename;                // the file containing the xml information 
var pkgXML=new HW_XML();
var out=pkgXML.toObject({file:the_filename}); // read & parse xml data
 if (out.error()) 
   return({Status:'',Timestamp:''});

var Status=pkgXML.findEntities({name:'Status'});        // get status
var Status=typeof(Status.result[0].contents[0])=='undefined'?'':Status.result[0].contents[0];
var Timestamp=pkgXML.findEntities({name:'Timestamp'}); // get timestamp
var Timestamp=typeof(Timestamp.result[0].contents[0])=='undefined'?'':Timestamp.result[0].contents[0];

return({Status:Status,Timestamp:Timestamp});
}
//-------------------------------------------------------------------------------------------------
function parseParams() {
  //
  // parses the hwjs command line parameters 
  //
  // In: command line parameters
  //
  //   -hwhost    (opt) - specifies the hyperwave server, default = localhost
  //   -hwport    (opt) - the port number, default value = 418 (HGCSP)
  //   -user      (req) - username
  //   -password  (req) - password
  //   -help            - usage
  //
  // Out: return object {hwhost,hwport,username,password}
  //

var host = "localhost" ; // default hostname
var port = 418;          // default portnumber
var user="";             
var password="";

var x = new OptionParser(argv,{"-hwhost":1,"-hwport":2,"-user":3,"-password":4,"-help":5}) ;

while (x.more())
{
   switch (x.option())
   {
     case 1: // -hwhost
         if (x.more() && !x.nextIsOption())
         {
           host = x.parameter() ;
         }
         else
         {
            write("-hwhost requires parameter\n") ;
            exit(1);
         }
      break;
     case 2: // -hwport
         if (x.more() && !x.nextIsOption())
         {
           port = x.parameter() ;
         }
         else
         {
            write("-hwport requires parameter\n") ;
            exit(1);
         }
      break;
     case 3: // -user
         if (x.more() && !x.nextIsOption())
         {
           user = x.parameter() ;
         }
         else
         {
            write("-user requires parameter\n") ;
            exit(1);

         }
      break;
     case 4:// -password
         if (x.more() && !x.nextIsOption())
         {
           password = x.parameter() ;
         }
         else
         {
            write("-password requires parameter\n") ;
            exit(1);

         }
      break;
     case 5:// -help
       writeHelp(); exit();
     break;
         
      case -1 :
         write ("Error: unknown option\n"); writeHelp();
         exit();
      break ;
      case -2 :
         write ("Error: ambiguous commandline option:", x.ambiguous(),"\n") ;
         exit();
      break ;
      default:
         write ("Internal error\n") ;
         exit();
      break;
   }
}       
return ({hwhost:host,hwport:port,username:user,password:password});
}
//----------------------------------------------------------------------------------------------------
function writeFile(fileObject,content) {
var result="\n";
//writelnError(fileObject.name);
out=fileObject.open("w"); // create file 
 if (!out) result+='File "'+fileObject.name+'" could not be created.\n';
out=fileObject.write("\n"+content+"\n");
 if (!out) result+='Error: could not write to file "'+fileObject.name+'".\n';
out=fileObject.close();
 if (!out) result+='Error: could not close file "'+fileObject.name+'".\n';
return result;
}
//------------------------------------------------------------------------------------------------------------------
function portal_convertVersion (version_str)
{
  if (typeof version_str != "string" ||
    version_str == "")
    return 0;

  version_str = version_str.replace(/v/i,"");
  if (version_str.indexOf('_')!=-1)
    var arr = version_str.split('_');
  else
    var arr = version_str.split('.');
  var ret = 0;
  for (var i=0; i<3; i++)
  {
    ret *= 100;
    if (typeof(arr[i])!="undefined")
      ret += parseInt(arr[i]);
  }
  return ret;
}

//------------------------------------------------------------------------------------------------------------------
function portal_eq (obja, objb) 
{
  var obja = obja || {};
  var objb = objb || {};

  if (obja.Product != objb.Product) 
    return {result:false, reason:"Name"};
  if (portal_convertVersion(obja.Version) != portal_convertVersion(objb.Version)) 
    return {result:false, reason:"Version"};
  if (portal_convertVersion(obja.SP) != portal_convertVersion(objb.SP)) 
    return {result:false, reason:"SP"};

  return {result:true};
}

//------------------------------------------------------------------------------------------------------------------
function portal_ge (obja, objb) 
{
  var obja = obja || {};
  var objb = objb || {};

  if (obja.Product != objb.Product) 
    return {result:false, reason:"Name"};
  if (portal_convertVersion(obja.Version) < portal_convertVersion(objb.Version)) 
    return {result:false, reason:"Version"};
  if (portal_convertVersion(obja.SP) < portal_convertVersion(objb.SP)) 
    return {result:false, reason:"SP"};

  return {result:true};
}

//------------------------------------------------------------------------------------------------------------------
function portal_extractInstallInfo (productXML)
{
  var retobj;
  var installed_products=productXML.findEntities({name:'product'}); // get 'Package' entity

        // loop through found products
  for (var i=0; i<installed_products.result.length; i++)
  {
    var contents = installed_products.result[i].contents;
          // look for products of interest
    if (typeof contents[0]=="string" &&
      ((contents[0].search(/^\s*(Hyperwave Information Server)\s$/)!=-1 && (typeof retobj!="object" || typeof retobj["Product"]!="string")) || 
        contents[0].search(/^\s*(Hyperwave Information Portal)\s$/)!=-1))  // remove leading and trailing spaces
    {
      retobj = {};
      retobj["Product"] = RegExp.$1;  // product name
      retobj["product"] = RegExp.$1;  // product name
      for (var j=1; j<contents.length; j++)
      {
        if (contents[j].name != "service_pack")  // store product information in return object
        {
          retobj[contents[j].name] = contents[j].contents[0]; // version, release_date
          if (contents[j].name=="version") retobj["Version"] = contents[j].contents[0];
        }
        else
        {
                // store service_pack information in object retobj.service_pack
          retobj[contents[j].name] = {};
          for (var k=0; k<contents[j].contents.length; k++)
          {
            if (typeof contents[j].contents[k] == "object")
            {
              retobj[contents[j].name][contents[j].contents[k].name] = contents[j].contents[k].contents[0];
                    // store version of service_pack into property SP to make
                    // comparisons with Dependency information in packages easier
              if (contents[j].contents[k].name=="version")
                retobj["SP"] = contents[j].contents[k].contents[0]; // service pack version
            }
          }
        }
      }
      if (retobj["Product"]=="Hyperwave Information Portal") break;  // product found -> exit loop
    }
  }
  return typeof retobj == "object" ? retobj : null;
}

//------------------------------------------------------------------------------------------------------------------
function portal_setServicePackInfo (productsinfo, servicepackinfo)
{
        // replaces placeholder "%%Today%%" with the current date
        // necessary for tag "Install_Date"
  var spobj = (typeof servicepackinfo.value[0] == "object" ? servicepackinfo.value[0] : servicepack.value);

  for (var k=1; k<spobj.contents.length; k++)
  {
    if (spobj.contents[k].contents[0]=="%%Today%%")
      spobj.contents[k].contents[0] = outDate(new Date(),"YYYY-MM-DD");
  }

  var installed_products = productsinfo.findEntities({name:'product'});  // get installed products
        // find the product to which the service pack belongs
  for (var i=0; i<installed_products.result.length; i++)
  {
    var contents = installed_products.result[i].contents;
          // remove leading and trailing spaces
    var found = (typeof contents[0]=="string" ? contents[0].search(/^\s*(.*)\s*$/) : -1);  
    if (found != -1)
    {
      var product = RegExp.$1;
      if (product == servicepackinfo.attributes["Product"])
      {
        var set = false;  
        for (var j=1; j<contents.length; j++)
        {
                // if a service pack already exists replace this information
          if (contents[j].name == "service_pack")  
          {
            contents[j] = spobj;
            set = true;
            break;  // servive pack found -> exit loop
          }
        }
              // if no service pack has existed yet add a new entry
        if (!set)
          contents[contents.length] = spobj;
        break;  // product found -> exit loop
      }
    }
  }
  return productsinfo;
}
//------------------------------------------------------------------------------------------------------------------
