/** ****************************************************************************
 * Function findExperts uses this function to rank the authors.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-12-14 
 *
 **/
function sortArray(a,b) { 
  return b[1] - a[1]; 
}


/** ****************************************************************************
 * Function findExperts uses this function to calculate the overall score for an
 * expert.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-12-14 
 *
 **/
function calculateScoreForAuthor(theauthor, theexpertobject) {
//      out = server.find ({keyquery: 'Author='+theauthor,
//                          languages: ['en','ge'],
//                          startIndex: 0,
//                          numberOfObjectsToGet: 1});
//      if (out.error.error()){
//        writelnError('Author: '+theauthor+' numberofdocs: 5000');
//      }    
//      writelnError('Author: '+theauthor+' numberofdocs: '+out.numberOfObjectsFound);
  return theexpertobject.score;
} 


/** ****************************************************************************
 * Function findExperts uses this function to calculate the score for a single 
 * document.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-12-14 
 *
 **/
function calculateScoreForDocument(theobject) {
  var score = Math.floor(theobject.Score);
  var theobject_year = dateConv(theobject.TimeModified || theobject.TimeCreated, HWDate, HW_UI_Date.full).year;
  var currentyear = new Date().getYear();
  if (currentyear < 1000) 
    currentyear += 1900;
  var difference = currentyear - theobject_year;

  switch (difference) {
  case 0: 
    return score;
    break;
  default: 
    return score/difference;
  }
}


/** ****************************************************************************
 * Users specified in the array EXCLUDE_USERS_ARRAY are not experts. This array
 * is defined in file findexpert.js in the directory apps/hip/v1.0/config.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-12-14 
 *
 **/
function authorIsInExcludeList(theauthor) {
  for (var id =0; id < findexpert.EXCLUDE_USERS_ARRAY.length; id++) {
    if (findexpert.EXCLUDE_USERS_ARRAY[id] == theauthor)
      return true;
  }
  return false;
}


/** ****************************************************************************
 * Normalizes the score to a scale of 100.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-12-15 
 *
 **/
function normalizeArray(thearray) {
  themax = thearray[0][1];
  for (var id in thearray) {
    thearray[id][1] = Math.ceil(thearray[id][1]/(themax+1)*100);
  }
}

/** ****************************************************************************
 * Find experts takes a search string and performs a combined full text and 
 * key words query (in attribute Title). The authors of the documents in the search
 * result are ranked and an array of arrays with information for each author is
 * returned (number of experts is limited to 20).
 *
 * Array:[['author','score','number of relevant documents'][..]..]
 *
 * @author: Moritz Markus
 *
 * @created: 1999-12-14 
 * @modified: 1999-12-17
 * @comment: Use a special user account to gather information about the experts.
 **/
function findExperts(thequery) {
  var f=new File(findexpert.READUSERFILE);
  if (f.open("r")) {
    var theusername=f.readln();
    var thepassword=f.readln();
    f.close();
  }
  if (theusername && thepassword) {
    var theserver = new HW_API_Server(hw_getServerIPAddress());
    var out = theserver.identify({username:theusername,password:thepassword});
    if (out.error.error())  //identify was not successful; do not switch account
      var theserver = server;
  }
  else
    var theserver = server;
  var expertsobject = {};
  var returnobject = {};
  returnobject.result = [];
  returnobject.error = new HW_API_Error();
  var searchlanguages = [];
  for (var id in HW_Settings.LANGUAGES) {
    searchlanguages[searchlanguages.length] = HW_Settings.LANGUAGES[id].shortn;
  }

  var cVal='(Title='+thequery+')||(Name='+thequery+')||(Keyword='+thequery+')';
  while(place("extended.search.customindex.next_entry"))
    cVal+="||("+place("extended.search.customindex.entry.attribute")+"="+thequery+")";
  
  out = theserver.find({fulltextquery: thequery, 
                     keyquery: cVal,                 
                     // scope: ['rootcollection'],
                     languages: searchlanguages});
  if (out.error.error()){
    returnobject.error = out.error;
    return returnobject;
  }    

  // create info object for each author of documents in the search
  for (var id in out.objects) {
    author = out.objects[id].Author;
    documentscore = calculateScoreForDocument(out.objects[id]);
    if (expertsobject[author]) {
      expertsobject[author].numberofrelevantdocuments++;
      expertsobject[author].score = expertsobject[author].score + documentscore;
    }
    else {
      expertsobject[author] = {numberofrelevantdocuments:1,score:documentscore};
    }
  }
  
  var numberofexperts = 0;
  for (var author in expertsobject) {
    if (!authorIsInExcludeList(author)) {
      numberofexperts++;
      var score = calculateScoreForAuthor(author,expertsobject[author]);
      var numberofrelevantdocuments = expertsobject[author].numberofrelevantdocuments;
      returnobject.result[returnobject.result.length] = [author, score, numberofrelevantdocuments]; 
    }
    if (numberofexperts == 20) break;
  }

  returnobject.result.sort(sortArray);
  if (returnobject.result.length) 
    normalizeArray(returnobject.result);
  return returnobject;
}

