/**
********************************************************************************
     Start object HW_XML
********************************************************************************
 **/

/** ****************************************************************************
 * This function is the prototype of the HW_XML class. It initializes the 'document entity' object.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @returns: the HW_XML object
 *
 **/
function HW_XML() {
  this.xmlroot = {};
  this.xmlroot.type = 'document entity';
  this.xmlroot.name = 'root';
  this.xmlroot.attributes = [];
  this.xmlroot.contents = [];
}


HW_XML.prototype.toObject = HW_XML_toObject;
HW_XML.prototype.toString = HW_XML_toString;
HW_XML.prototype.insertEntity = HW_XML_insertEntity;
HW_XML.prototype.removeEntity = HW_XML_removeEntity;
//HW_XML.prototype.copyEntity = HW_XML_copyEntity;   muss wirklich kopiert werden!
HW_XML.prototype.moveEntity = HW_XML_moveEntity;
HW_XML.prototype.insertMultipleEntities = HW_XML_insertMultipleEntities;
HW_XML.prototype.getChildren = HW_XML_getChildren;
HW_XML.prototype.getParent = HW_XML_getParent;
HW_XML.prototype.findEntities = HW_XML_findEntities;
HW_XML.prototype.getContentsLength = HW_XML_getContentsLength;
HW_XML.prototype.getAttributeValue = HW_XML_getAttributeValue;
HW_XML.prototype.insertAttribute = HW_XML_insertAttribute;
HW_XML.prototype.removeAttribute = HW_XML_removeAttribute;
HW_XML.prototype.createEntity = HW_XML_createEntity;


/** ****************************************************************************
 * This function takes a string and creates an XML parse tree.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 * 
 * @modified: 1999-12-06
 * @comment: functionality added that stops parsing when a defined number of 
 *           entities has been created
 *
 * @param parameters.xmlstring: the string to parse
 * @param parameters.file:      path and name of the file that contains the string
 *                              you have to specify either xmlstring or file
 * @param parameters.limitentititesbyname.maxentities: 
 *
 * @param parameters.limitentititesbyname.maxentities: 
 *
 *
 * @returns an HW_API_Error object
 * 
 **/
function HW_XML_toObject(parameters) {
  if (parameters.limitentitiesbyname) {
    var maxentities = parameters.limitentitiesbyname.maxentities || null;
    var limitentityname = parameters.limitentitiesbyname.entityname || null;
  }
  else {
    maxentities = null;
    limitentitiesbyname = null;
  }

  var entitycount = 0;
  if (maxentities && !limitentityname)
    return new HW_API_Error('Error: No entity name for entity limit specified'); 
  if (limitentityname && !maxentities)
    return new HW_API_Error('Error: No entity limit for entity '+limitentityname+' specified'); 
    
  if (parameters.xmlstring) {
    xmlstring = parameters.xmlstring;
  }
  else {
    if (parameters.file) {
      var xmlfile = new File(parameters.file);
      if (!(xmlfile.open('r'))) {
        return new HW_API_Error('Error: Could not open file '+parameters.file);
      }
      xmlstring = xmlfile.read(xmlfile.getLength());
      xmlfile.close();
      xmlstring = xmlstring.substring(xmlstring.indexOf('<'),xmlstring.length);
    }
    else {
      if (parameters.url) {
        // HTTP_HOST holds the host name or IP address of the proxy (if there is one)
        // or the name/IP address of the target server
        var HTTP_HOST = parameters.http_host || undefined;
        // HTTP_PORT holds the port number of the proxy (if there is one)
        // or the port address of the target server
        var HTTP_PORT = parameters.http_port || 80;

        var newssocket = new Socket();  // socket object that connects to a server
        var result = [];

        if (!newssocket.connect(HTTP_HOST,HTTP_PORT)) {
          return new HW_API_Error('Error: Could not connect to server'); 
        }
      
//        newssocket.write('GET http://'+parameters.url+' HTTP/1.1\n');
        //        writelnError('GET '+parameters.url+' HTTP/1.0\n');
        newssocket.write('GET '+parameters.url+' HTTP/1.0\n');
        newssocket.write('\n');

        while (!newssocket.eof()) {
          tempread = newssocket.read(4096);
          if (tempread == ''  &&!newssocket.eof() && newssocket.error()) {
//            writelnError('errorno: '+newssocket.error());
            newssocket.close();
            return new HW_API_Error('Error: Could not retrieve file'); 
          }
          result[result.length] = tempread;
        }
        
//          while (!newssocket.eof()) {
//            tempread = newssocket.read(4096);
//            if (newssocket.error()) {
//              writelnError('-------> '+newssocket.error()+'---');
//              newssocket.close();
//              return new HW_API_Error('Error: Could not retrieve file'); 
//            }
//            result[result.length] = tempread;
//          }
        newssocket.close();
        xmlstring = result.join('');
        delete result;
      }
    }
  }

  xmlstring = xmlstring.substring(xmlstring.indexOf('<'),xmlstring.lastIndexOf('>')+1);
  xmlstring = xmlstring.replace(/\t\n/g,'\n');
  xmlstring = xmlstring.replace(/\t/g,'\n');
  
  var root = new Object();
  root.name = 'root';
  root.type = 'document entity';
  root.attributes = new Array();
  root.contents = new Array();
  
  var stack = new Array(); // data structure that helps building the xml object
  stack.push(root);
  
  var currentpos = 0;      // stores the current position in the string xmlstring
  var xmllength = xmlstring.length;
  
  while (currentpos < xmllength) {
    tagstart = xmlstring.indexOf('<',currentpos);
    tagend = xmlstring.indexOf('>',tagstart)+1;
    
    // stop parsing when end of file or entitylimit is reached
    if (tagstart == -1 || entitycount > maxentities) {
      this.xmlroot = root;
      return new HW_API_Error();
    }
    
    contentspos = stack[stack.length-1].contents.length; // the current tag object position + 1
    
    if (tagstart != currentpos) { // take text and store it in the contents-array
      var plaintext = xmlstring.substring(currentpos,tagstart);
      if (/\S/.test(plaintext)) { 
        // remove leading and trailing spaces and whitespace used to format the content in the xml string
        plaintext = plaintext.replace(/^\s+/g,'');
        plaintext = plaintext.replace(/$\s+/g,'');
        plaintext = plaintext.replace(/\s+\n/g,'\n');
        plaintext = plaintext.replace(/\n\s+/g,'\n');
        plaintext = plaintext.replace(/&gt;/g,'>');
        plaintext = plaintext.replace(/&lt;/g,'<');
        plaintext = plaintext.replace(/&quot;/g,'"');
        plaintext = plaintext.replace(/&apos;/g,"'");
        plaintext = plaintext.replace(/&amp;/g,'&');
        stack[stack.length-1].contents[contentspos] = plaintext;
        contentspos ++;
      }
    }    
    switch (xmlstring[tagstart+1]) {
    case '/':  // check if it is a closing tag
      if (!RegExp(xmlstring.substring(tagstart+2,tagend-1)+'\s*').test(stack[stack.length-1].name)) {
        //        writeln(xmlstring);
        return new HW_API_Error('Error: Entity '+stack[stack.length-1].name+' has no closing tag'); 
      }
      stack.pop();  // pop the last element from the stack (its finished)
      break;
      
    case '!':  // handle comments and CDATA sections
      var currententity = {};
      currententity.attributes = [];
      currententity.contents = [];
      if (xmlstring.substring(tagstart+1,tagstart+4) == '!--') {
        tagend = xmlstring.indexOf('-->',currentpos)+3;
        currententity.type = 'comment'; 
        currententity.contents[0] = xmlstring.substring(tagstart+4,tagend-3);
      }
      else {
        if (xmlstring.substring(tagstart+1,tagstart+9) == '![CDATA[') {
          tagend = xmlstring.indexOf(']]>',currentpos)+3;
          currententity.type = 'cdata'; 
          currententity.contents[0] = xmlstring.substring(tagstart+9,tagend-3);
        }
        else {
          if (xmlstring.substring(tagstart+1,tagstart+9).toLowerCase() == '!doctype') {
            currententity.type = 'doctype'; 
            currententity.contents[0] = xmlstring.substring(tagstart+10,tagend-1);
          }
        }
      }
      stack[stack.length-1].contents[contentspos] = currententity;
      break;
      
    case '?':  // handle processing instructions and xml declarations
      tagend = xmlstring.indexOf('?>',currentpos)+2;
      var currententity = {};
      currententity.attributes = [];
      currententity.contents = [];
      if (xmlstring.substring(tagstart+1,tagstart+6).toLowerCase() == '?xml ') {
        currententity.type = 'xml declaration'; 
        currententity.contents[0] = xmlstring.substring(tagstart+6,tagend-2);
      }
      else {
        currententity.type = 'pi'; 
        currententity.contents[0] = xmlstring.substring(tagstart+2,tagend-2);
      }
      stack[stack.length-1].contents[contentspos] = currententity;
      break;
      
    default:
      var currententity = {};
      currententity.attributes = [];
      currententity.contents = [];
      currententity.type = 'element'; 
      
      var re_tagname = /([\w:]+)\s*/g;
      re_tagname.lastIndex = tagstart+1;
      currententity.name = re_tagname.exec(xmlstring)[1];
      
      if (currententity.name == limitentityname) 
        entitycount++;
      
      if (re_tagname.lastIndex < tagend-2) {  // parse attributes
        var re_attributename = /(\S+)\s*=\s*/g;
        re_attributename.lastIndex = tagstart+1;
        do {
          var attributename = re_attributename.exec(xmlstring)[1];
          var attributedelimiter = xmlstring[re_attributename.lastIndex];
          if (attributedelimiter != '"' && attributedelimiter != "'") {
            return new HW_API_Error('Error: Attribute '+attributename+' is not enclosed in quotes or double quotes.');
          }
          var re_attributevalue = new RegExp(attributedelimiter+'([^'+attributedelimiter+']*)'+attributedelimiter+'\s*','g');
          re_attributevalue.lastIndex = re_attributename.lastIndex;
          var attributevalue = re_attributevalue.exec(xmlstring)[1];
          if (attributevalue.indexOf('<') != -1) {
            return new HW_API_Error('Error: Attribute '+attributename+' is not enclosed in quotes or double quotes.');
          }
          //              attributevalue = attributevalue.replace(/&gt;/g,'>');
          attributevalue = attributevalue.replace(/&lt;/g,'<');
          attributevalue = attributevalue.replace(/&amp;/g,'&');
          if (attributedelimiter == "'") 
            attributevalue = attributevalue.replace(/&quot;/g,'"');
          else
            attributevalue = attributevalue.replace(/&apos;/g,"'");
          attributename.lastIndex = re_attributevalue.lastIndex;
          currententity.attributes[attributename] = attributevalue;
          if (attributevalue.indexOf('>') != -1) 
            tagend = xmlstring.indexOf('>',re_attributevalue.lastIndex)+1;
        } while (re_attributevalue.lastIndex < tagend-3);
      }
      
      stack[stack.length-1].contents[contentspos] = currententity;  
      if (xmlstring[tagend-2] != '/')   //push the new object on the stack if it is not empty
        stack.push(stack[stack.length-1].contents[contentspos]);     
      
    }
    currentpos = tagend ;
  }
  this.xmlroot = root;
  return new HW_API_Error();
} 


/** ****************************************************************************
 * This function takes an XML parse tree and creates a string.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param parameters.entity:           starting entity for the creation of the 
 *                                     string; by default the whole XML parse tree 
 *                                     will be converted to a string
 * @param parameters.filename:         path and name of the file the string will be
 *                                     stored in (optional)
 * @param parameters.fileaccessrights: can be 'w' (overwrite existing file) or 'w+' 
 *                                     (append to existing file, default value)
 *                                     
 * @returns out.error:     a HW_API_Error object
 * @returns out.xmlstring: the string
 *
 **/
function HW_XML_toString(parameters) {
  var entity = parameters.entity || this.xmlroot;
  var filename = parameters.filename || undefined;
  var fileaccessrights = parameters.fileaccessrights || 'w';
  var out = {};
  var xmlarray = [];
  var xmlindex = 0;
  
  function createstring(entity,indent) {
    switch (entity.type.toLowerCase()) {
    case 'element':
      xmlarray[xmlindex++] = indent+'<'+entity.name;
      for (var attrib in entity.attributes)  {
        var attributevalue = entity.attributes[attrib];
        attributevalue = attributevalue.replace(/&/g,'&amp;'); //<-----------------------------
        attributevalue = attributevalue.replace(/</g,'&lt;');
        attributevalue = attributevalue.replace(/>/g,'&gt;');
        attributevalue = attributevalue.replace(/"/g,'&quot;'); //")
        xmlarray[xmlindex++] = ' '+attrib+'="'+attributevalue+'"';
      }
      if (entity.contents.length == 0)  xmlarray[xmlindex++] = '/';
      xmlarray[xmlindex++] = '>\n';
      
      for (var contid = 0; contid < entity.contents.length; contid++) {
        if (typeof entity.contents[contid] != 'undefined') {
          if (typeof entity.contents[contid] != 'object') {
            if (typeof entity.contents[contid] == 'string') 
              var content = entity.contents[contid];
            else 
              if (typeof entity.contents[contid] == 'number') 
                var content = content.toString();
            content = content.replace(/</g,'&lt;');
            content = content.replace(/>/g,'&gt;');
            content = content.replace(/&/g,'&amp;');
            if ((content.substring(content.lastIndexOf('\n'),content.length)).match(/\S/))  
              xmlarray[xmlindex++] = indent +'  '+content+'\n';
            else
              xmlarray[xmlindex++] = indent +'  '+content;
          }
          else {
            createstring(entity.contents[contid],indent+'  ');
          }
          //                  writeln(entity.name+': '+entity.contents.length);
          //                  if (cont == entity.contents.length-1) {
          //                      xmlarray[xmlindex++] = indent+'</'+entity.name+'>\n';
          //                  }
        }
      }
      if (entity.contents.length != 0)  
        xmlarray[xmlindex++] = indent+'</'+entity.name+'>\n';
      break;
    case 'comment':
      xmlarray[xmlindex++] = indent+'<!--'+entity.contents[0]+'-->\n';
      break;
    case 'document entity':
      for (var id in entity.contents)
        createstring(entity.contents[id], indent);
      break;
    case 'pi':
      xmlarray[xmlindex++] = indent+'<?'+entity.contents[0]+'?>\n';
      break;
    case 'cdata':
      xmlarray[xmlindex++] = indent+'<![CDATA['+entity.contents[0]+']]>\n';
      break;
    case 'xml declaration':
      xmlarray[xmlindex++] = indent+'<?xml '+entity.contents[0]+'?>\n';
      break;
    case 'doctype':
      xmlarray[xmlindex++] = indent+'<!doctype '+entity.contents[0]+'>\n';
      break;
    default:
      xmlarray[xmlindex++] = 'unknown entity type! -- '+entity.type+'\n';
    }
  }
  
  createstring(entity,'');
  out.xmlstring = xmlarray.join('');
  out.error = new HW_API_Error();
  delete xmlarray;
  if (filename) {
    var xmlfile = new File(filename);
    if (xmlfile.open(fileaccessrights)) {
      xmlfile.write(out.xmlstring);
      xmlfile.close();
    }
    else out.error = new HW_API_Error('Error: could not write file');
  }
  return out;
} 


/** ****************************************************************************
 * This function takes an entity and inserts it at the specified position into the XML parse tree.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param parameters.parententity: the parent entity of the new object
 * @param parameters.position: The position in the contents array of the parent entity that will contain the new entity. An existing entity on this position will be overwritten! (will be altered in the next version).
 * @param parameters.entity: the entity object to be inserted
 *
 * @returns an HW_API_Error object
 * 
 **/
function HW_XML_insertEntity(parameters) {
  var parententity = parameters.parententity;
  var position = parameters.position;
  var newentity = parameters.entity;
  var mode = parameters.mode;
  parententity.contents[position] = [];
  parententity.contents[position].type = newentity.type;
  parententity.contents[position].name = newentity.name;
  parententity.contents[position].attributes = newentity.attributes;
  parententity.contents[position].contents = newentity.contents;
  return new HW_API_Error();
}


/** ****************************************************************************
 * This function removes an entity from the XML parse tree.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param: parameters.parententity: the parent entity 
 * @param: parameters.entity: the entity object that should be removed
 *
 * @returns an HW_API_Error object
 *
 **/
function HW_XML_removeEntity(parameters) {
  var parententity = parameters.parententity;
  var entity = parameters.entity;
  for (var id in parententity.contents) {
    if (parententity.contents[id] == entity) {
      delete parententity.contents[id];
      return new HW_API_Error();
    }
  }
  return new HW_API_Error('Error: Specified entity not found');
}


/** ****************************************************************************
 * This function moves an entity within the XML parse tree.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param: parameters.sourceparententity: the source parent entity 
 * @param: parameters.destinationparententity: the destination parent entity 
 * @param: parameters.destinationposition: the position in the destination parent contents array 
 * @param: parameters.entity: the entity object that should be moved 
 *
 * @returns an HW_API_Error object
 *
 **/
function HW_XML_moveEntity(parameters) {
  var sourceparententity = parameters.sourceparententity;
  var destinationparententity = parameters.destinationparententity;
  var destinationposition = parameters.destinationposition;
  var entity = parameters.entity;
  for (var id in sourceparententity.contents) {
    if (sourceparententity.contents[id] == entity) {
      destinationparententity.contents[destinationposition] = entity;
      delete sourceparententity.contents[id];
      return new HW_API_Error();
    }
  }
  return new HW_API_Error('Entity not found');
}


/** ****************************************************************************
 * This function inserts an array of entities into the XML parse tree.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param: parameters.parententity: the entity object that should store the array of entities
 * @param: parameters.entityarray: an array of entity objects
 * @param: parameters.mode: mode = overwrite replaces existing children of the parent entity with the new entity objects
 *                          mode = append adds the new entity objects to the existing children of the parent object (default) 
 *
 * @returns an HW_API_Error object
 * 
 **/
function HW_XML_insertMultipleEntities(parameters) {
  var parententity = parameters.parententity;
  var entityarray = parameters.entityarray;
  var mode = parameters.mode || 'append';
  
  if (mode == 'overwrite' || mode == 'append') {
    if (mode == 'overwrite') 
      parententity.contents = [];
    for (var id in entityarray) {
      parententity.contents[parententity.contents.length] = entityarray[id];
    }
    return new HW_API_Error();
  }
  else 
    return new HW_API_Error('Unknown mode');
}


/** ****************************************************************************
 * This function returns the children of an entity. useful?
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param: parameters.entity: the entity
 *
 * @returns an array of entity objects or character data strings
 *
 **/
function HW_XML_getChildren(parameters) {
  var entity = parameters.entity;
  var out = {};
  out.error = new HW_API_Error();
  out.children = entity.contents;
  return out;
}


/** ****************************************************************************
 * This function returns the parent of an entity. useful?
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param: parameters.entity: the entity
 *
 * @returns out.parent: the parent entity object
 * @returns out.error: an HW_API_Error object
 *
 **/
function HW_XML_getParent(parameters) {
  var entity = parameters.entity;
  var out = {};
  out.error = new HW_API_Error();
  
  function checkChildren_internal(theEntity) {
    if (theEntity.contents.length > 0) {
      for (var id in theEntity.contents) {
        if (typeof theEntity.contents[id] == 'object') {
          if (theEntity.contents[id] == entity) {
            out.parent = theEntity;
            return;
          }
          else {
            checkChildren_internal(theEntity.contents[id]);
          }
        }
      }
    }
  }
  checkChildren_internal(this.xmlroot);
  
  if (typeof (out.parent != 'undefined')) {
    return out;
  }
  else {
    out.error = new HW_API_Error('no parent found');
    return out;
  }
}


/** ****************************************************************************
 * This function performs a search and returns an array of entity objects that match the given search parameters.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param: parameters.type: return only objects with the given type 
 * @param: parameters.name: return only objects with the given name 
 * @param: parameters.attribute.name: return only objects with the given attribute name 
 * @param: parameters.attribute.value: return only objects with the given attribute value (works only in combination with parameters.attribute.name)
 *         It is possible to combine search parameters. 
 *
 * @returns out.result: an array of entity objects that match the given search query
 * @returns out.error: an HW_API_Error object
 *
 **/
function HW_XML_findEntities(parameters) {
  var type = parameters.type || undefined;
  var name = parameters.name || undefined;
  var out = {};
  out.error = new HW_API_Error();
  out.result = [];
  
  if (typeof parameters.attribute == 'object') {
    var attributename = parameters.attribute.name || undefined;
    var attributevalue = parameters.attribute.value || undefined;
  }
  else {
    var attributename = undefined;
    var attributevalue = undefined;
  }
  function findEntities_internal(theEntity) {
    var index = 0;
    if (!type) index++;
    else if (type == theEntity.type) index++;
    if (!name) index++;
    else if (name == theEntity.name) index++;
    if (!attributename) index += 2;
    else {
      if (typeof theEntity.attributes[attributename] != 'undefined') {
        index++;
      }
      if (!attributevalue) index++;
      else if (theEntity.attributes[attributename] == attributevalue) index++;
    }
    
    if (index == 4) out.result[out.result.length] = theEntity;
    
    if (theEntity.contents.length > 0) { // drill down
      for (var id in theEntity.contents) {
        if (typeof theEntity.contents[id] == 'object') {
          findEntities_internal(theEntity.contents[id]);
        }
      }
    }
  }
  findEntities_internal(this.xmlroot);
  if (out.result.length != 0) {
    return out;
  }
  else {
    return out;
  }
}


/** ****************************************************************************
 * This function takes an HW_XML_Entity and returns the length of the contents array.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param: parameters.entity: the entity
 *
 * @returns the length of the contents array
 *
 **/
function HW_XML_getContentsLength(parameters) {
  var entity = parameters.entity;
  return entity.contents.length;
}


/** ****************************************************************************
 * This function returns the value of an attribute.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param parameters.entity: the entity
 * @param parameters.attributename: the name of the attribute
 *
 * @returns out.value: the value of the specified attribute
 * @returns out.error: a HW_API_Error object
 * 
 **/
function HW_XML_getAttributeValue(parameters) {
  var entity = parameters.entity;
  var attributename = parameters.attributename;
  var out = {};
  out.error = new HW_API_Error();
  
  if (typeof entity.attributes[attributename] != 'undefined') {
    out.value = entity.attributes[attributename];
    return out;
  }
  else {
    out.error = new HW_API_Error('Attribute not defined');
    return out;
  }
}


/** ****************************************************************************
 * This function inserts an attribute into the attributes array of the entity object.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param parameters.entity: the entity
 * @param parameters.attribute.name: the name of the new attribute
 * @param parameters.attribute.value: the value of the new attribute
 *
 * @returns out.error: a HW_API_Error object
 * 
 **/
function HW_XML_insertAttribute(parameters) {
  var entity = parameters.entity;
  var attributename = parameters.attribute.name;
  var attributevalue = parameters.attribute.value;
  
  entity.attributes[attributename] = attributevalue;
  return new HW_API_Error();
}


/** ****************************************************************************
 * This function removes a given attribute from an entity object.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param parameters.entity: the entity
 * @param parameters.attributename: the attribute to be removed
 *
 * @returns out.error:     a HW_API_Error object
 * 
 **/
function HW_XML_removeAttribute(parameters) {
  var entity = parameters.entity;
  var attributename = parameters.attributename;
  
  if (typeof entity.attributes[attributename] != 'undefined') {
    delete entity.attributes[attributename];
    return new HW_API_Error();
  }
  else {
    return new HW_API_Error('Attribute not defined');
  }
}


/** ****************************************************************************
 * This function serves as a 'constructor' for a new entity object.
 *
 * @author: Moritz Markus
 *
 * @created: 1999-11-03 
 *
 * @param parameters.type: the type of the new entity object; default:element
 * @param parameters.name: the name of the new entity object; default:empty string
 * @param parameters.attributes: the attributes array of the new entity object; default: empty array
 * @param parameters.contents: the contents array of the new entity object; default: empty array
 *
 * @returns out.error:     a HW_API_Error object
 * @returns out.xmlstring: the Entities array
 * 
 **/
function HW_XML_createEntity(parameters) {
  out = {};
  out.type = parameters.type || 'element';
  out.name = parameters.name || '';
  out.attributes = parameters.attributes || [];
  out.contents = parameters.contents || [];
  return out;
}
