/** ****************************************************************************
 * Hyperwave Information Portal                                                *
 * Package UnInstall Script                                                    *
 *******************************************************************************
 * There are two ways to execute the uninstall script:                         *
 *                                                                             *
 * (1) From the commandline using the hwjs                                     *
 * (2) From the wavetemplates (frontend to the uninstall script)               *
 *                                                                             *
 * @author: Gartler Michael                                                    *
 *                                                                             *
 *******************************************************************************/

var _version='Version 1, March 28th, 2000';

//-------------------------------------------------------------------------------------------------
function writeHelp() {
  //
  // displays help message (when used from commandline)
  //
  writeln('\n\
Hyperwave Information Portal ('+_version+')\nPackage Uninstall Script\n\n\
Usage: hwjs uninstall.js <Options> \n\n\
Options\n\t-hwhost name \t\t(opt) name of Hyperwave server host (default: localhost)\n\
\t-hwport port \t\t(opt) port number to connect to (default: 418)\n\
\t-user username\t\t(req) specify username\n\
\t-password password\t(req) specify password\n\n'); 
}
//------------------------------------------------------------------------------------------------------------------
function addZero(x) {
  x='0'+x;
  x=x.substr(-2);
  return(x);
}
//------------------------------------------------------------------------------------------------------------------
function getDateString(dobj) {
  //
  //
  //
  var year=dobj.getFullYear()+"";
  year=year.substr(2);
  var month=dobj.getMonth()+1;
  month=addZero(month);
  var day=dobj.getDate();
  day=addZero(day);
  var hour=dobj.getHours();
  hour=addZero(hour);
  var minute=dobj.getMinutes();
  minute=addZero(minute);
  var second=dobj.getSeconds();
  second=addZero(second);

  var res=month+day+year+'.'+hour+minute+second;
  return res;
}
//------------------------------------------------------------------------------------------------------------------
function create_saveDir(params) {
  //
  // creates a backup directory ('save_dir') in a specified directory. 
  //
  // In: params object
  //   - path : the parent directory of the save_dir
  //
  // Out: status object
  //   - result  : [true|false] (true if operation was successful)
  //   - statusLog : details, error messages etc.
  //
  var statusLog='Backup:\t'
  var the_path=params.path;
  var dobj=new Date();
  var the_name='backup.'+getDateString(dobj);
  var dir_obj=new Directory(the_path+the_name);
  var out=dir_obj.create();
  if(!out)
    return({result:false,statusLog:statusLog+' could not create directory '+dir_obj.name+'.\n',backupdir:''});
  else
    return({result:true,statusLog:statusLog+' directory '+dir_obj.name+' created.\n',backupdir:the_path+the_name});
}
//------------------------------------------------------------------------------------------------------------------
function delDir(params) {
  //
  // deletes all content of a directory which is older than a given date (timestamp)
  //
  // In: params object
  //     - timestamp : a date (HWDate format: YYYY/MM/DD hh:mm:ss)
  //     - dir_name  : delete content of this directory
  //
  // Out: status object
  //   - result  : [true|false] (true if operation was successful)
  //   - statusLog : details, error messages etc.
  //
  var timestamp=params.timestamp;
  var dir_name=params.dir_name+'/';
  var dir=new Directory(dir_name);
  var res=params.result;
  var statusLog=params.statusLog;

  if (!dir.open()) {
    res=false;
    statusLog+='\t\t\tError: could not open directory '+dir.name+'\n';
    return({result:res,statusLog:statusLog}); }

  for (;;) {

    do {
      var name=dir.read();}
    while((name=='.')||(name=='..'));

    if (!name)
      break;

    var f=new File(dir.name+name);

    if (f.isDirectory()&&(name!='.')&&(name!='..')) {
      var out=delDir({dir_name:dir.name+name,timestamp:timestamp,result:res,statusLog:statusLog});
      if (!out.result)
        return({result:out.result,statusLog:out.statusLog});
      else
        statusLog=out.statusLog; }
    else {
      var TimeModified=new Date(f.modified()*1000);
      TimeModified=outDate(TimeModified,HWDate.full);
      TimeModified=outDate(dateConv(TimeModified,HWDate,true),HWDate.full); 
        // convert to GMT using HWDate format
      if(!f.remove()) {
        res=false;
        statusLog+='\n\t\t\tError: could not remove file '+f.name;
        return({result:res,statusLog:statusLog}); }
    }
  }
  dir.close();
  if(!dir.remove()) {
    res=false;
    statusLog+='\n\t\t\tError: could not remove directory '+dir.name;
    return({result:res,statusLog:statusLog}); }

  return({result:res,statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function removeDirectory(params) {
  //
  // deletes all content of directory older than a given date using function delDir
  // moves directory to the backup dir (savedir)
  //
  // In : params object
  //      - timestamp
  //      - dir_name
  //      - savedir_name
  //
  // Out: status object
  //   - result  : [true|false] (true if operation was successful)
  //   - statusLog : details, error messages etc.
  //

  var isBackup=params.isBackup;
  var isRemove=params.isRemove;
  var timestamp=params.timestamp;
  var dir_name=HW_Settings.HTML_DIR+'/'+params.dir_name;
  var dir_obj=new Directory(dir_name);
  var savedir_name=params.savedir_name+'/'+params.dir_name.replace(/\//g,'_');
  var statusLog='';
  if (isRemove) statusLog='Removed Directory:\t'+dir_name+' ... ';
  else statusLog='Backup Directory:\t'+dir_name+' ... ';
  var res=true;

  if (isBackup)
    if(!dir_obj.move(savedir_name)) {
      statusLog+='failed.\n\t\t\tError: could not move '+dir_name+' to '+savedir_name;
      return({result:false,statusLog:statusLog}); }
    else
      statusLog+='ok.';

  if(!isBackup) {
    var out=delDir({dir_name:dir_name,timestamp:timestamp,result:res,statusLog:''});
    if (out.result)
      statusLog+='ok.';
    else {
      statusLog+='failed.'+out.statusLog;
      res=false;}
  }

  return({result:res,statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function findUniqueName(basedir,name) {
  //
  //
  //
  name=name.substr(0,250);
  var fobj=new File(basedir+'/'+name);
  var index=1;
  var name2="";
  while (fobj.exists()) {
    name2=name+'~'+index;
    fobj=new File(basedir+'/'+name2);
    index++ }

  if (name2=="")
    return basedir+'/'+name;
  else
    return basedir+'/'+name2;
}
//------------------------------------------------------------------------------------------------------------------
function removeObject(params) {
  //
  // deletes an object from the database; saves all data modified after a given data (timestamp) to the savedir
  //
  // In: params object
  //      - object
  //      - timestamp
  //      - backupdir
  //
  // Out: status object
  //   - result  : [true|false] (true if operation was successful)
  //   - statusLog : details, error messages etc.
  //

  var isRemove=params.isRemove;
  var isBackup=params.isBackup;

  var statusLog='';
  
  var the_object=params.object;
  var the_backupdir=params.backupdir;
  var the_backupdestination=the_object.replace(/\//g,'_');

  if (isBackup) {
    var uniqueBackupDirName=findUniqueName(the_backupdir,the_backupdestination);
    var dobj=new Directory(uniqueBackupDirName);
    out=dobj.create();
      if (!out)
      writeln('could not create dir '+dobj.name);}

  var the_timestamp=params.timestamp;
  var res=true;
  if (isRemove) 
    statusLog='Removed Collection:\t'+the_object+' ... ';
  else
    statusLog='Backup Collection:\t'+the_object+' ... ';

  //writeln('savedir : ',the_backupdir);
  //writeln('timestamp : ',the_timestamp);
  //writeln('object : ',the_object);

  /*
  var out=server.find({scope:[the_object],keyquery:'TimeModified>'+the_timestamp,languages:['en']});
    if (out.error.error()) {
      statusLog+='failed.\n\tFind Error:\n';
      for (var x in out.error)
        statusLog+='\t'+out.error[x].description("en")+'\n';
       return({result:false,statusLog:statusLog}); }

  var objs=out.objects;
  var save_list='';
  for (var i=0; i<objs.length; i++) {
    var the_GOid=objs[i].GOid.replace(' ','_');
    //writeln(the_GOid,' --->',objs[i].TimeModified); 
    save_list+=' '+the_GOid; }
  save_list=save_list.substr(1);
  save_list='"'+save_list+'"';
  //writeln(save_list);
  */

  //if (save_list!='""') {
  //  var exec='hwxexport -hwhost '+params.hwhost+' -hwport '+params.hwport+' -identify "'+params.username+' '+params.password+'" -hwxfile '+the_backupdir+'/'+the_hwxfilename+' -objects '+save_list;         

  var exec='APIhwdownload -hwhost '+params.hwhost+' -hwport '+params.hwport+' -identify "'+params.username+' '+params.password+'" -parent '+the_object+' -targetdir '+the_backupdir+'/'+the_backupdestination;

  //  writelnError(exec);

  if (isBackup) {
    var sysres=system (exec);                           // execute hwximport
    //    writelnError('--->ready !!');
    if (sysres!=0) {                                    // check result of hwximport
      res=false;
      statusLog+='failed.\n\tError: could not hwxexport';
      return({result:res,statusLog:statusLog}); }
  }

  if (isRemove) {
    var out=server.remove({objectidentifier:the_object}); // remove object
    if (out.error.error()) {
      statusLog+='failed.\nError:\n';
      for (var x in out.error)
        statusLog+='\t'+out.error[x].description("en")+'\n';
      return({result:false,statusLog:statusLog}); 
    }
  }
  statusLog+='ok.';

  return ({result:res,statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function removeFile(params) {
  var filename         = params.filename;
  var savedir_name     = params.savedir_name;
  var isBackup         = params.isBackup;
  var isRemove         = params.isRemove;

  var statusLog = 'Remove File:\t\t' + filename + ' ... ';

  var file = new File(HW_Settings.HTML_DIR + '/' + filename);

  if (isBackup) {
    var index = filename.lastIndexOf('/');
    var fname = index == -1 ? filename : filename.substring(index+1);

    if (!file.copy(savedir_name + '/' + fname)) {
      statusLog += 'failed.\n\t\tError: Cannot copy "' + filename + '" to "' + savedir_name + '".';
      return ({result:false, statusLog:statusLog});
    }
  }

  if (isRemove)
    if (!file.remove()) {
      statusLog += 'failed.\n\t\tError: Cannot remove "' + filename + '".';
      return ({result:false, statusLog:statusLog});
    }

  statusLog += 'ok.';
  return ({result:true, statusLog:statusLog});
}
//------------------------------------------------------------------------------------------------------------------
function removeInclude(params) {
  var package_name     = params.package_name;
  var package_version = params.package_version;
  var is_place_include = params.is_place_include;
  var full_include     = params.include_file;

  var statusLog = 'Remove Include:\t\t' + full_include + ' ... ';

  // remove include directive from 'hwincludes.html'
  var VERSION_DIR = HW_Settings.VERSION_DIR || "v1.0";
  var hwincludes_name = HW_Settings.HTML_DIR + '/' + VERSION_DIR + '/hwincludes.html';

  var file = new File(hwincludes_name);
  if(!file.open('r')) {
    statusLog += 'failed.\n\tError: Cannot open "' + hwincludes_name + '" for reading.';
    return ({result:false, statusLog:statusLog});
  }

  if (is_place_include)
    var lines = "%%# PKGINC (PLACE)(" + package_name + " [" + package_version + "])(" + full_include + ") %%\n" +
                "%%include \"" + full_include + "\"%%\n";
  else
    var lines = "%%# PKGINC (JS)(" + package_name + " [" + package_version + "])(" + full_include + ") %%\n" +
                "<server src=\"file://" + full_include + "\">\n";

  var content = file.read(file.getLength());
  file.close();

  var index = content.indexOf(lines);
  if (index != -1) {
    file = new File(hwincludes_name);
    if(!file.open('w')) {
      statusLog += 'failed.\n\tError: Cannot open "' + hwincludes_name + '" for writing.';
      return ({result:false, statusLog:statusLog});
    }

    file.write(content.substring(0, index) + content.substring(index + lines.length));

    file.close();
  }

  // delete or backup the file
  var out = removeFile({filename:full_include, savedir_name: params.savedir_name,
                        isBackup: params.isBackup, isRemove: params.isRemove});
  if (!out.result) {
    statusLog += 'failed.\n\tError: ' + out.statusLog;
    return ({result:false, statusLog:statusLog});
  }
  else {
    statusLog += 'ok.\n' + out.statusLog;
    return ({result:true, statusLog:statusLog});
  }
}
//------------------------------------------------------------------------------------------------------------------
function uninstall(api,inargs) {
  //
  // uninstalls a package
  //
  // In: api, inargs
  // 
  // inargs:
  //  - path     : the path to the package directory
  //  - hwhost   : the name of the hyperwave server (default=localhost)
  //  - hwport   : the port number (default=418)
  //  - username : the username
  //  - password : the password
  // 
  // Out: result
  //
  // result:
  //  - status (string)     :  status of installation; [installed,failed,exists,no_hwx] 
  // 
  var HW_Settings   = inargs.HW_Settings;

  var isBackup      = inargs.isBackup;
  var isRemove      = inargs.isRemove;

  var pkgInfo       = parsePkgXML({filename:inargs.path+"pkg.xml"});    // read info from pkg.xml
  var the_Name      = pkgInfo.packageId.Name;                           // the package name
  var the_Version   = pkgInfo.packageId.Version;                        // the package version
  var the_ForceBackup = (pkgInfo.packageId.ForceBackup=="true") ? true : false;   // do backup in any case
  isBackup = isBackup || the_ForceBackup;
  var the_DeInstall = (pkgInfo.packageId.DeInstall=="false") ? false : true;       // allow deinstallation
  var the_Title     = pkgInfo.Title[0].value;                           // the package title
  var the_Resources = pkgInfo.Resource;                                 // the package resources
  var obj_toRemove  = pkgInfo.Uninstall;                                // the objects to remove 
  var res_path      = inargs.path;                                      // 

  var the_username  = inargs.username;                                  // username
  var the_password  = inargs.password;                                  // password
  var the_hostname  = inargs.hwhost;                                    // hostname
  var the_port      = inargs.hwport;                                    // port number

  var pathInstalled = inargs.path+'installed.xml';                      // path to 'installed.xml' file
  var pathStatus    = inargs.path+'status.log';                         // path to 'status.log' file
  var Finstalled    = new File(pathInstalled);                          // 'installed'
  var Fstatus       = new File(pathStatus);                             // 'status.log'

  var inst;                                                             // installation status
  var file_result   ="";                                                // results of filesystem actions
  var inst_Date     = new Date();

  var xml='<?xml version="1.0"?>\n';

  var statusObj={result:true,statusLog:'Hyperwave Information Portal ('+_version+')\nPackage Uninstall Status Report\n'+inst_Date.toLocaleString()+'\n\nPackage:\t'+the_Name+'\nVersion:\t'+the_Version+'\n\n'};

  var install_obj=parseInstalledXML({filename:pathInstalled});          // parse installed.xml
  var the_Status=install_obj.Status;                                    // get installation status (if available)
  var the_Timestamp=install_obj.Timestamp;                              // get timestamp
  the_Timestamp=outDate(dateConv(the_Timestamp,HWDate,true),HWDate.full); 

  /*
  if (the_Status!='installed')
    return ({status:false,error:'This package has already been removed'});

    */

  if (!the_DeInstall)
  {
    statusObj.result = false;
    statusObj.statusLog += 'Error: It is not allowed to deinstall this package.\n\n';
    writeFile(Fstatus,statusObj.statusLog);
  }
  else
  {
    if (typeof(obj_toRemove)=='undefined') {
      the_error='Error in the Uninstall Information (no objects defined to remove)';
      statusObj.statusLog+=the_error;
      writeFile(Fstatus,statusObj.statusLog);
      writeFile(Finstalled,xml+'<Status>uninstall_error</Status>\n<Timestamp></Timestamp>');
      return ({status:false,error:the_error}); }
    
    if (isBackup) {
      out=create_saveDir({path:res_path}); // create backup directory 
      statusObj.statusLog+=out.statusLog;
      var the_backupdir=out.backupdir;
      if (!out.result) {
        the_error='Error: could not create backup directory';
        writeFile(Fstatus,statusObj.statusLog);
        writeFile(Finstalled,xml+'<Status>uninstall_error</Status>\n<Timestamp></Timestamp>');
        return ({status:false,error:the_error}); 
      }
    }
    else 
      var the_backupdir='';
    
    for (var i=0; i<obj_toRemove.length; i++) {                   // loop thru all objects to remove
      
      var obj=obj_toRemove[i].attributes.Object;                  // name of the object to remove
      var type=obj_toRemove[i].attributes.Type;
      
      switch(type) {
        case 'his' : out=removeObject({ object:obj, 
                                        timestamp:the_Timestamp,
                                        backupdir:the_backupdir,
                                        hwhost:the_hostname,
                                        hwport:the_port,
                                        username:the_username,
                                        password:the_password,
                                        isBackup:isBackup,
                                        isRemove:isRemove }); 
                     if (!out.result)
                       statusObj.result=false;
                     if (out.statusLog!='')
                       statusObj.statusLog+='\n'+out.statusLog; 
                     break;

        case 'dir' : out=removeDirectory({ dir_name:obj, 
                                           savedir_name:the_backupdir,
                                           timestamp:the_Timestamp,
                                           isBackup:isBackup,
                                           isRemove:isRemove }); 
                     if (!out.result)
                       statusObj.result=false;
                     if (out.statusLog!='')
                       statusObj.statusLog+='\n'+out.statusLog;
                     break;
                     
        case 'file' : out = removeFile({ filename:obj, 
                                         savedir_name:the_backupdir, 
                                         isBackup:isBackup, 
                                         isRemove:isRemove});
                      if (!out.result)
                        statusObj.result=false;
                      if (out.statusLog!='')
                        statusObj.statusLog+='\n'+out.statusLog;
                      break;

        case 'placeinclude' : out = removeInclude({ package_name:the_Name, 
                                                    package_version:the_Version, 
                                                    is_place_include:true,
                                                    include_file:obj, 
                                                    savedir_name:the_backupdir, 
                                                    isBackup:isBackup, 
                                                    isRemove:isRemove});
                              if (!out.result)
                                statusObj.result=false;
                              if (out.statusLog!='')
                                statusObj.statusLog+='\n'+out.statusLog;
                              break;
                              
        case 'jsinclude' : out = removeInclude({package_name:the_Name, 
                                                package_version:the_Version, 
                                                is_place_include:false,
                                                include_file:obj, 
                                                savedir_name:the_backupdir, 
                                                isBackup:isBackup, 
                                                isRemove:isRemove});
                           if (!out.result)
                             statusObj.result=false;
                           if (out.statusLog!='')
                             statusObj.statusLog+='\n'+out.statusLog;
                           break;
      }
    }
    writeFile(Fstatus,statusObj.statusLog);
  }

  if (statusObj.result)
    writeFile(Finstalled,xml+'<Status>uninstalled</Status><Timestamp></Timestamp>');
  else  
    writeFile(Finstalled,xml+'<Status>uninstall_error</Status>\n<Timestamp></Timestamp>');

   
  return({status:statusObj.result,error:""});                                 // return installation result
}
//===================================================================================================
if (typeof(server)=='undefined') {                                    // Execution context is HWJS... 
  //
  // check if execution context is hwjs (there's no server object in hwjs)
  // this is needed to determine wheter the install scripts is called from commandline or via templates
  //

  HW_Settings={HTML_DIR:"../../../../"};

  var dir = new Directory(HW_Settings.HTML_DIR);
  if (!dir.open())
  {
    writeln('\n\nError: Cannot include necessary files. Cannot open directory '+HW_Settings.HTML_DIR+'.\n\n');
  }
  else
  {
    var version_dir = "";
    for (;;)
    {
      var name = dir.read();
      if (!name)
        break;
      if (name.search(/^v.\..+$/)!=-1 && name!="v4.1") 
      {
        version_dir = name;
        break;
      }
    }
    dir.close();

    if (version_dir == "")
    {
      writeln('\n\nError: Cannot include necessary files. Cannot find version directory.\n\n');
    }
    else
    {
      HW_Settings.VERSION_DIR = version_dir;

      HW_UI_ConvertGMTLocalGMT=true;

      include(HW_Settings.HTML_DIR+"/"+HW_Settings.VERSION_DIR+"/lib/datefunctions.js");
      include(HW_Settings.HTML_DIR+"/"+HW_Settings.VERSION_DIR+"/lib/xmllib.js");                               // xml parser
      include(HW_Settings.HTML_DIR+"/"+HW_Settings.VERSION_DIR+"/components/portal/code/install_lib.js");       // required lib functions
 
      var args=parseParams();
   
      if (args.username=='') { 
        writeln('\nPlease specify an username.'); writeHelp(); exit(); }  // Error: no username specified

      if (args.password=='') { 
        writeln('\nPlease specify a password.'); writeHelp(); exit();}    // Error: no password specified
      
      server=new HW_API_Server({host:args.hwhost,port:args.hwport});
      out=server.identify({username:args.username,password:args.password}); // identify
      if (out.error.error()) {
        writeln('Error: Could not identify (wrong username or password)\n');
        exit(); }
      
      args.HW_Settings=HW_Settings;
      args.isBackup=true;
      args.isRemove=true;
      args.path="";
      var result=uninstall("",args);
      
      writeln('\n');
      
      if (result.status)
        writeln('The Package has been successfully uninstalled. See file "status.log" for more details.');
      else
        writeln('Failed to uninstall the package:\n'+result.error+'\nSee file "status.log" for more details.');
    }
  }
      
  writeln('\n');
}
